@@ -10,6 +10,9 @@
 #define USETITER_H
 
 #include "unicode/utypes.h"
+
+#if U_SHOW_CPLUSPLUS_API
+
 #include "unicode/uobject.h"
 #include "unicode/unistr.h"
 
@@ -30,9 +33,7 @@ class UnicodeString;
  * code points or ranges have been returned, it returns the
  * multicharacter strings of the UnicodeSet, if any.
  *
- * This class is not intended to be subclassed.  Consider any fields
- *  or methods declared as "protected" to be private.  The use of
- *  protected in this class is an artifact of history.
+ * This class is not intended for public subclassing.
  *
  * <p>To iterate over code points and strings, use a loop like this:
  * <pre>
@@ -57,25 +58,23 @@ class UnicodeString;
  *   }
  * }
  * </pre>
+ *
+ * To iterate over only the strings, start with <code>skipToStrings()</code>.
+ *
  * @author M. Davis
  * @stable ICU 2.4
  */
-class U_COMMON_API UnicodeSetIterator : public UObject {
-
- protected:
-
+class U_COMMON_API UnicodeSetIterator U_FINAL : public UObject {
     /**
      * Value of <tt>codepoint</tt> if the iterator points to a string.
      * If <tt>codepoint == IS_STRING</tt>, then examine
      * <tt>string</tt> for the current iteration result.
-     * @stable ICU 2.4
      */
     enum { IS_STRING = -1 };
 
     /**
      * Current code point, or the special value <tt>IS_STRING</tt>, if
      * the iterator points to a string.
-     * @stable ICU 2.4
      */
     UChar32 codepoint;
 
@@ -86,7 +85,6 @@ class U_COMMON_API UnicodeSetIterator : public UObject {
      * iterating over code points using <tt>next()</tt>, or if
      * <tt>codepoint == IS_STRING</tt>, then the value of
      * <tt>codepointEnd</tt> is undefined.
-     * @stable ICU 2.4
      */
     UChar32 codepointEnd;
 
@@ -94,7 +92,6 @@ class U_COMMON_API UnicodeSetIterator : public UObject {
      * If <tt>codepoint == IS_STRING</tt>, then <tt>string</tt> points
      * to the current string.  If <tt>codepoint != IS_STRING</tt>, the
      * value of <tt>string</tt> is undefined.
-     * @stable ICU 2.4
      */
     const UnicodeString* string;
 
@@ -167,13 +164,30 @@ class U_COMMON_API UnicodeSetIterator : public UObject {
      */
     const UnicodeString& getString();
 
+    /**
+     * Skips over the remaining code points/ranges, if any.
+     * A following call to next() or nextRange() will yield a string, if there is one.
+     * No-op if next() would return false, or if it would yield a string anyway.
+     *
+     * @return *this
+     * @stable ICU 70
+     * @see UnicodeSet#strings()
+     */
+    inline UnicodeSetIterator &skipToStrings() {
+        // Finish code point/range iteration.
+        range = endRange;
+        endElement = -1;
+        nextElement = 0;
+        return *this;
+    }
+
     /**
      * Advances the iteration position to the next element in the set, 
      * which can be either a single code point or a string.  
      * If there are no more elements in the set, return false.
      *
      * <p>
-     * If <tt>isString() == TRUE</tt>, the value is a
+     * If <tt>isString() == true</tt>, the value is a
      * string, otherwise the value is a
      * single code point.  Elements of either type can be retrieved
      * with the function <tt>getString()</tt>, while elements of
@@ -194,7 +208,7 @@ class U_COMMON_API UnicodeSetIterator : public UObject {
     /**
      * Returns the next element in the set, either a code point range
      * or a string.  If there are no more elements in the set, return
-     * false.  If <tt>isString() == TRUE</tt>, the value is a
+     * false.  If <tt>isString() == true</tt>, the value is a
      * string and can be accessed with <tt>getString()</tt>.  Otherwise the value is a
      * range of one or more code points from <tt>getCodepoint()</tt> to
      * <tt>getCodepointeEnd()</tt> inclusive.
@@ -202,7 +216,7 @@ class U_COMMON_API UnicodeSetIterator : public UObject {
      * <p>The order of iteration is all code points ranges in sorted
      * order, followed by all strings sorted order.  Ranges are
      * disjoint and non-contiguous.  The value returned from <tt>getString()</tt>
-     * is undefined unless <tt>isString() == TRUE</tt>.  Do not mix calls to
+     * is undefined unless <tt>isString() == true</tt>.  Do not mix calls to
      * <tt>next()</tt> and <tt>nextRange()</tt> without calling
      * <tt>reset()</tt> between them.  The results of doing so are
      * undefined.
@@ -239,71 +253,58 @@ class U_COMMON_API UnicodeSetIterator : public UObject {
      *
      * @stable ICU 2.4
      */
-    virtual UClassID getDynamicClassID() const;
+    virtual UClassID getDynamicClassID() const override;
 
     // ======================= PRIVATES ===========================
 
- protected:
+private:
 
     // endElement and nextElements are really UChar32's, but we keep
     // them as signed int32_t's so we can do comparisons with
     // endElement set to -1.  Leave them as int32_t's.
     /** The set
-     * @stable ICU 2.4
      */
     const UnicodeSet* set;
     /** End range
-     * @stable ICU 2.4
      */
     int32_t endRange;
     /** Range
-     * @stable ICU 2.4
      */
     int32_t range;
     /** End element
-     * @stable ICU 2.4
      */
     int32_t endElement;
     /** Next element
-     * @stable ICU 2.4
      */
     int32_t nextElement;
-    //UBool abbreviated;
     /** Next string
-     * @stable ICU 2.4
      */
     int32_t nextString;
     /** String count
-     * @stable ICU 2.4
      */
     int32_t stringCount;
 
     /**
      *  Points to the string to use when the caller asks for a
      *  string and the current iteration item is a code point, not a string.
-     *  @internal
      */
     UnicodeString *cpString;
 
     /** Copy constructor. Disallowed.
-     * @stable ICU 2.4
      */
-    UnicodeSetIterator(const UnicodeSetIterator&); // disallow
+    UnicodeSetIterator(const UnicodeSetIterator&) = delete;
 
     /** Assignment operator. Disallowed.
-     * @stable ICU 2.4
      */
-    UnicodeSetIterator& operator=(const UnicodeSetIterator&); // disallow
+    UnicodeSetIterator& operator=(const UnicodeSetIterator&) = delete;
 
     /** Load range
-     * @stable ICU 2.4
      */
-    virtual void loadRange(int32_t range);
-
+    void loadRange(int32_t range);
 };
 
 inline UBool UnicodeSetIterator::isString() const {
-    return codepoint == (UChar32)IS_STRING;
+    return codepoint < 0;
 }
 
 inline UChar32 UnicodeSetIterator::getCodepoint() const {
@@ -317,4 +318,6 @@ inline UChar32 UnicodeSetIterator::getCodepointEnd() const {
 
 U_NAMESPACE_END
 
+#endif /* U_SHOW_CPLUSPLUS_API */
+
 #endif
