@@ -1,6 +1,10 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
-* Copyright (C) 2003, International Business Machines Corporation and others. All Rights Reserved.
-********************************************************************************
+******************************************************************************
+* Copyright (C) 2003-2013, International Business Machines Corporation
+* and others. All Rights Reserved.
+******************************************************************************
 *
 * File HEBRWCAL.H
 *
@@ -9,7 +13,7 @@
 *   Date        Name        Description
 *   05/13/2003  srl          copied from gregocal.h
 *   11/26/2003  srl          copied from buddhcal.h
-********************************************************************************
+******************************************************************************
 */
 
 #ifndef HEBRWCAL_H
@@ -84,7 +88,7 @@ public:
    * Useful constants for HebrewCalendar.
    * @internal
    */
-  enum EEras {
+  enum Month {
     /** 
      * Constant for Tishri, the 1st month of the Hebrew year. 
      */
@@ -176,19 +180,12 @@ public:
      */
     HebrewCalendar(const HebrewCalendar& source);
 
-    /**
-     * Default assignment operator
-     * @param right    the object to be copied.
-     * @internal
-     */
-    HebrewCalendar& operator=(const HebrewCalendar& right);
-
     /**
      * Create and return a polymorphic copy of this calendar.
      * @return    return a polymorphic copy of this calendar.
      * @internal
      */
-    virtual Calendar* clone(void) const;
+    virtual HebrewCalendar* clone() const override;
     
 public:
     /**
@@ -201,7 +198,7 @@ public:
      *           same class ID. Objects of other classes have different class IDs.
      * @internal
      */
-    virtual UClassID getDynamicClassID(void) const;
+    virtual UClassID getDynamicClassID(void) const override;
 
     /**
      * Return the class ID for this class. This is useful only for comparing to a return
@@ -214,7 +211,7 @@ public:
      * @return   The class ID for all objects of this class.
      * @internal
      */
-    static UClassID getStaticClassID(void);
+    static UClassID U_EXPORT2 getStaticClassID(void);
 
     /**
      * return the calendar type, "hebrew".
@@ -222,7 +219,7 @@ public:
      * @return calendar type
      * @internal
      */
-    virtual const char * getType() const;
+    virtual const char * getType() const override;
 
 
     // Calendar API
@@ -238,11 +235,11 @@ public:
      *                previously set in the time field is invalid, this will be set to
      *                an error status.
      */
-    virtual void add(UCalendarDateFields field, int32_t amount, UErrorCode& status);
+    virtual void add(UCalendarDateFields field, int32_t amount, UErrorCode& status) override;
     /**
      * @deprecated ICU 2.6 use UCalendarDateFields instead of EDateFields
      */
-    inline virtual void add(EDateFields field, int32_t amount, UErrorCode& status) { add((UCalendarDateFields)field, amount, status); }
+    virtual void add(EDateFields field, int32_t amount, UErrorCode& status) override;
 
 
     /**
@@ -256,7 +253,7 @@ public:
      *                an error status.
      * @internal
      */
-    virtual void roll(UCalendarDateFields field, int32_t amount, UErrorCode& status);
+    virtual void roll(UCalendarDateFields field, int32_t amount, UErrorCode& status) override;
 
     /**
      * (Overrides Calendar) Rolls up or down by the given amount in the specified field.
@@ -269,8 +266,12 @@ public:
      *                an error status.
      * @deprecated ICU 2.6. Use roll(UCalendarDateFields field, int32_t amount, UErrorCode& status) instead.
 `     */
-    virtual void roll(EDateFields field, int32_t amount, UErrorCode& status);
+    virtual void roll(EDateFields field, int32_t amount, UErrorCode& status) override;
 
+    /**
+     * @internal 
+     */
+    static UBool isLeapYear(int32_t year) ;
 
  protected:
 
@@ -295,7 +296,7 @@ public:
      * <code>LEAST_MAXIMUM</code>, or <code>MAXIMUM</code>
      * @internal
      */
-    virtual int32_t handleGetLimit(UCalendarDateFields field, ELimitType limitType) const;
+    virtual int32_t handleGetLimit(UCalendarDateFields field, ELimitType limitType) const override;
 
     /**
      * Return the number of days in the given month of the given extended
@@ -304,7 +305,7 @@ public:
      * implementation than the default implementation in Calendar.
      * @internal
      */
-    virtual int32_t handleGetMonthLength(int32_t extendedYear, int32_t month) const;
+    virtual int32_t handleGetMonthLength(int32_t extendedYear, int32_t month) const override;
 
     /**
      * Return the number of days in the given extended year of this
@@ -313,7 +314,7 @@ public:
      * default implementation in Calendar.
      * @stable ICU 2.0
      */
-    virtual int32_t handleGetYearLength(int32_t eyear) const;
+    virtual int32_t handleGetYearLength(int32_t eyear) const override;
     /**
      * Subclasses may override this method to compute several fields
      * specific to each calendar system.  These are:
@@ -329,7 +330,7 @@ public:
      * a calendar with the specified Julian/Gregorian cutover date.
      * @internal
      */
-    virtual void handleComputeFields(int32_t julianDay, UErrorCode &status);
+    virtual void handleComputeFields(int32_t julianDay, UErrorCode &status) override;
     /**
      * Return the extended year defined by the current fields.  This will
      * use the UCAL_EXTENDED_YEAR field or the UCAL_YEAR and supra-year fields (such
@@ -338,7 +339,7 @@ public:
      * @return the extended year
      * @internal
      */
-    virtual int32_t handleGetExtendedYear();
+    virtual int32_t handleGetExtendedYear() override;
     /**
      * Return the Julian day number of day before the first day of the
      * given month in the given extended year.  Subclasses should override
@@ -353,9 +354,16 @@ public:
      * @internal
      */
     virtual int32_t handleComputeMonthStart(int32_t eyear, int32_t month,
-                                                   UBool useMonth) const;
+                                                   UBool useMonth) const override;
 
 
+    /**
+     * Validate a single field of this calendar.
+     * Overrides Calendar::validateField(int) to provide
+     * special handling for month validation for Hebrew calendar.
+     * @internal
+     */
+    virtual void validateField(UCalendarDateFields field, UErrorCode &status) override;
 
  protected:
 
@@ -368,69 +376,26 @@ public:
    *           false, otherwise.
    * @internal
    */
-  virtual UBool inDaylightTime(UErrorCode& status) const;
+  virtual UBool inDaylightTime(UErrorCode& status) const override;
 
-    /**
-     * Returns TRUE because the Hebrew Calendar does have a default century
-     * @internal
-     */
-    virtual UBool haveDefaultCentury() const;
+  /**
+   * Returns true because the Hebrew Calendar does have a default century
+   * @internal
+   */
+  virtual UBool haveDefaultCentury() const override;
 
-    /**
-     * Returns the date of the start of the default century
-     * @return start of century - in milliseconds since epoch, 1970
-     * @internal
-     */
-    virtual UDate defaultCenturyStart() const;
+  /**
+   * Returns the date of the start of the default century
+   * @return start of century - in milliseconds since epoch, 1970
+   * @internal
+   */
+  virtual UDate defaultCenturyStart() const override;
 
-    /**
-     * Returns the year in which the default century begins
-     * @internal
-     */
-    virtual int32_t defaultCenturyStartYear() const;
-
- private: // default century stuff.
-    /**
-     * The system maintains a static default century start date.  This is initialized
-     * the first time it is used.  Before then, it is set to SYSTEM_DEFAULT_CENTURY to
-     * indicate an uninitialized state.  Once the system default century date and year
-     * are set, they do not change.
-     */
-    static UDate         fgSystemDefaultCenturyStart;
-
-    /**
-     * See documentation for systemDefaultCenturyStart.
-     */
-    static int32_t          fgSystemDefaultCenturyStartYear;
-
-    /**
-     * Default value that indicates the defaultCenturyStartYear is unitialized
-     */
-    static const int32_t    fgSystemDefaultCenturyYear;
-
-    /**
-     * start of default century, as a date
-     */
-    static const UDate        fgSystemDefaultCentury;
-
-    /**
-     * Returns the beginning date of the 100-year window that dates 
-     * with 2-digit years are considered to fall within.
-     */
-    UDate         internalGetDefaultCenturyStart(void) const;
-
-    /**
-     * Returns the first year of the 100-year window that dates with 
-     * 2-digit years are considered to fall within.
-     */
-    int32_t          internalGetDefaultCenturyStartYear(void) const;
-
-    /**
-     * Initializes the 100-year window that dates with 2-digit years
-     * are considered to fall within so that its start date is 80 years
-     * before the current time.
-     */
-    static void  initializeSystemDefaultCentury(void);
+  /**
+   * Returns the year in which the default century begins
+   * @internal
+   */
+  virtual int32_t defaultCenturyStartYear() const override;
 
  private: // Calendar-specific implementation
     /**
@@ -465,10 +430,6 @@ public:
      */
     int32_t yearType(int32_t year) const;
 
-    /**
-     * @internal 
-     */
-    static UBool isLeapYear(int32_t year) ;
     /**
      * @internal 
      */
