@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  **********************************************************************
- *   Copyright (C) 1999-2014, International Business Machines
+ *   Copyright (C) 1999-2016, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  **********************************************************************
  *   Date        Name        Description
@@ -146,12 +148,12 @@ public:
 
     virtual ~ParseData();
 
-    virtual const UnicodeString* lookup(const UnicodeString& s) const;
+    virtual const UnicodeString* lookup(const UnicodeString& s) const override;
 
-    virtual const UnicodeFunctor* lookupMatcher(UChar32 ch) const;
+    virtual const UnicodeFunctor* lookupMatcher(UChar32 ch) const override;
 
     virtual UnicodeString parseReference(const UnicodeString& text,
-                                         ParsePosition& pos, int32_t limit) const;
+                                         ParsePosition& pos, int32_t limit) const override;
     /**
      * Return true if the given character is a matcher standin or a plain
      * character (non standin).
@@ -192,9 +194,9 @@ const UnicodeFunctor* ParseData::lookupMatcher(UChar32 ch) const {
     const UnicodeFunctor* set = NULL;
     int32_t i = ch - data->variablesBase;
     if (i >= 0 && i < variablesVector->size()) {
-        int32_t i = ch - data->variablesBase;
-        set = (i < variablesVector->size()) ?
-            (UnicodeFunctor*) variablesVector->elementAt(i) : 0;
+        int32_t j = ch - data->variablesBase;
+        set = (j < variablesVector->size()) ?
+            (UnicodeFunctor*) variablesVector->elementAt(j) : 0;
     }
     return set;
 }
@@ -231,7 +233,7 @@ UBool ParseData::isMatcher(UChar32 ch) {
         UnicodeFunctor *f = (UnicodeFunctor*) variablesVector->elementAt(i);
         return f != NULL && f->toMatcher() != NULL;
     }
-    return TRUE;
+    return true;
 }
 
 /**
@@ -246,7 +248,7 @@ UBool ParseData::isReplacer(UChar32 ch) {
         UnicodeFunctor *f = (UnicodeFunctor*) variablesVector->elementAt(i);
         return f != NULL && f->toReplacer() != NULL;
     }
-    return TRUE;
+    return true;
 }
 
 //----------------------------------------------------------------------
@@ -346,7 +348,7 @@ RuleHalf::RuleHalf(TransliteratorParser& p) :
     post = -1;
     cursorOffset = 0;
     cursorOffsetPos = 0;
-    anchorStart = anchorEnd = FALSE;
+    anchorStart = anchorEnd = false;
     nextSegmentNumber = 1;
 }
 
@@ -362,7 +364,7 @@ RuleHalf::~RuleHalf() {
 int32_t RuleHalf::parse(const UnicodeString& rule, int32_t pos, int32_t limit, UErrorCode& status) {
     int32_t start = pos;
     text.truncate(0);
-    pos = parseSection(rule, pos, limit, text, UnicodeString(TRUE, ILLEGAL_TOP, -1), FALSE, status);
+    pos = parseSection(rule, pos, limit, text, UnicodeString(true, ILLEGAL_TOP, -1), false, status);
 
     if (cursorOffset > 0 && cursor != cursorOffsetPos) {
         return syntaxError(U_MISPLACED_CURSOR_OFFSET, rule, start, status);
@@ -401,7 +403,7 @@ int32_t RuleHalf::parseSection(const UnicodeString& rule, int32_t pos, int32_t l
     int32_t start = pos;
     ParsePosition pp;
     UnicodeString scratch;
-    UBool done = FALSE;
+    UBool done = false;
     int32_t quoteStart = -1; // Most recent 'single quoted string'
     int32_t quoteLimit = -1;
     int32_t varStart = -1; // Most recent $variableReference
@@ -509,7 +511,7 @@ int32_t RuleHalf::parseSection(const UnicodeString& rule, int32_t pos, int32_t l
         //------------------------------------------------------
         case ANCHOR_START:
             if (buf.length() == 0 && !anchorStart) {
-                anchorStart = TRUE;
+                anchorStart = true;
             } else {
               return syntaxError(U_MISPLACED_ANCHOR_START,
                                  rule, start, status);
@@ -527,7 +529,7 @@ int32_t RuleHalf::parseSection(const UnicodeString& rule, int32_t pos, int32_t l
                 int32_t segmentNumber = nextSegmentNumber++; // 1-based
                 
                 // Parse the segment
-                pos = parseSection(rule, pos, limit, buf, UnicodeString(TRUE, ILLEGAL_SEG, -1), TRUE, status);
+                pos = parseSection(rule, pos, limit, buf, UnicodeString(true, ILLEGAL_SEG, -1), true, status);
                 
                 // After parsing a segment, the relevant characters are
                 // in buf, starting at offset bufSegStart.  Extract them
@@ -569,7 +571,7 @@ int32_t RuleHalf::parseSection(const UnicodeString& rule, int32_t pos, int32_t l
                 int32_t bufSegStart = buf.length();
                 
                 // Parse the segment
-                pos = parseSection(rule, iref, limit, buf, UnicodeString(TRUE, ILLEGAL_FUNC, -1), TRUE, status);
+                pos = parseSection(rule, iref, limit, buf, UnicodeString(true, ILLEGAL_FUNC, -1), true, status);
                 
                 // After parsing a segment, the relevant characters are
                 // in buf, starting at offset bufSegStart.
@@ -596,7 +598,7 @@ int32_t RuleHalf::parseSection(const UnicodeString& rule, int32_t pos, int32_t l
                 if (pos == limit) {
                     // A variable ref character at the end acts as
                     // an anchor to the context limit, as in perl.
-                    anchorEnd = TRUE;
+                    anchorEnd = true;
                     break;
                 }
                 // Parse "$1" "$2" .. "$9" .. (no upper limit)
@@ -619,7 +621,7 @@ int32_t RuleHalf::parseSection(const UnicodeString& rule, int32_t pos, int32_t l
                         // end anchor then.  If this also doesn't work
                         // (if we see a following character) then signal
                         // an error.
-                        anchorEnd = TRUE;
+                        anchorEnd = true;
                         break;
                     }
                     pos = pp.getIndex();
@@ -702,7 +704,7 @@ int32_t RuleHalf::parseSection(const UnicodeString& rule, int32_t pos, int32_t l
         case SEGMENT_CLOSE:
             // assert(isSegment);
             // We're done parsing a segment.
-            done = TRUE;
+            done = true;
             break;
 
         //------------------------------------------------------
@@ -784,7 +786,7 @@ void RuleHalf::removeContext() {
         text.removeBetween(0, ante);
     }
     ante = post = -1;
-    anchorStart = anchorEnd = FALSE;
+    anchorStart = anchorEnd = false;
 }
 
 /**
@@ -796,10 +798,10 @@ UBool RuleHalf::isValidOutput(TransliteratorParser& transParser) {
         UChar32 c = text.char32At(i);
         i += U16_LENGTH(c);
         if (!transParser.parseData->isReplacer(c)) {
-            return FALSE;
+            return false;
         }
     }
-    return TRUE;
+    return true;
 }
 
 /**
@@ -811,10 +813,10 @@ UBool RuleHalf::isValidInput(TransliteratorParser& transParser) {
         UChar32 c = text.char32At(i);
         i += U16_LENGTH(c);
         if (!transParser.parseData->isMatcher(c)) {
-            return FALSE;
+            return false;
         }
     }
-    return TRUE;
+    return true;
 }
 
 //----------------------------------------------------------------------
@@ -889,7 +891,7 @@ void TransliteratorParser::parseRules(const UnicodeString& rule,
     uprv_memset(&parseError, 0, sizeof(parseError));
     parseError.line = parseError.offset = -1;
 
-    UBool parsingIDs = TRUE;
+    UBool parsingIDs = true;
     int32_t ruleCount = 0;
     
     while (!dataVector.isEmpty()) {
@@ -943,7 +945,7 @@ void TransliteratorParser::parseRules(const UnicodeString& rule,
         if (c == RULE_COMMENT_CHAR) {
             pos = rule.indexOf((UChar)0x000A /*\n*/, pos) + 1;
             if (pos == 0) {
-                break; // No "\n" found; rest of rule is a commnet
+                break; // No "\n" found; rest of rule is a comment
             }
             continue; // Either fall out or restart with next line
         }
@@ -973,13 +975,17 @@ void TransliteratorParser::parseRules(const UnicodeString& rule,
             
             if (!parsingIDs) {
                 if (curData != NULL) {
+                    U_ASSERT(!dataVector.hasDeleter());
                     if (direction == UTRANS_FORWARD)
                         dataVector.addElement(curData, status);
                     else
                         dataVector.insertElementAt(curData, 0, status);
+                    if (U_FAILURE(status)) {
+                        delete curData;
+                    }
                     curData = NULL;
                 }
-                parsingIDs = TRUE;
+                parsingIDs = true;
             }
 
             TransliteratorIDParser::SingleID* id =
@@ -1029,12 +1035,16 @@ void TransliteratorParser::parseRules(const UnicodeString& rule,
                     status = U_MEMORY_ALLOCATION_ERROR;
                     return;
                 }
+                U_ASSERT(idBlockVector.hasDeleter());
                 if (direction == UTRANS_FORWARD)
-                    idBlockVector.addElement(tempstr, status);
+                    idBlockVector.adoptElement(tempstr, status);
                 else
                     idBlockVector.insertElementAt(tempstr, 0, status);
+                if (U_FAILURE(status)) {
+                    return;
+                }
                 idBlockResult.remove();
-                parsingIDs = FALSE;
+                parsingIDs = false;
                 curData = new TransliterationRuleData(status);
                 // NULL pointer check
                 if (curData == NULL) {
@@ -1067,19 +1077,29 @@ void TransliteratorParser::parseRules(const UnicodeString& rule,
         tempstr = new UnicodeString(idBlockResult);
         // NULL pointer check
         if (tempstr == NULL) {
+            // TODO: Testing, forcing this path, shows many memory leaks. ICU-21701
+            //       intltest translit/TransliteratorTest/TestInstantiation
             status = U_MEMORY_ALLOCATION_ERROR;
             return;
         }
         if (direction == UTRANS_FORWARD)
-            idBlockVector.addElement(tempstr, status);
+            idBlockVector.adoptElement(tempstr, status);
         else
             idBlockVector.insertElementAt(tempstr, 0, status);
+        if (U_FAILURE(status)) {
+            return;
+        }
     }
     else if (!parsingIDs && curData != NULL) {
-        if (direction == UTRANS_FORWARD)
+        if (direction == UTRANS_FORWARD) {
             dataVector.addElement(curData, status);
-        else
+        } else {
             dataVector.insertElementAt(curData, 0, status);
+        }
+        if (U_FAILURE(status)) {
+            delete curData;
+            curData = nullptr;
+        }
     }
     
     if (U_SUCCESS(status)) {
@@ -1102,21 +1122,21 @@ void TransliteratorParser::parseRules(const UnicodeString& rule,
 
             for (int32_t j = 0; j < data->variablesLength; j++) {
                 data->variables[j] =
-                    ((UnicodeSet*)variablesVector.elementAt(j));
+                    static_cast<UnicodeFunctor *>(variablesVector.elementAt(j));
             }
             
             data->variableNames.removeAll();
-            int32_t pos = UHASH_FIRST;
-            const UHashElement* he = variableNames.nextElement(pos);
+            int32_t p = UHASH_FIRST;
+            const UHashElement* he = variableNames.nextElement(p);
             while (he != NULL) {
-                UnicodeString* tempus = (UnicodeString*)(((UnicodeString*)(he->value.pointer))->clone());
+                UnicodeString* tempus = ((UnicodeString*)(he->value.pointer))->clone();
                 if (tempus == NULL) {
                     status = U_MEMORY_ALLOCATION_ERROR;
                     return;
                 }
                 data->variableNames.put(*((UnicodeString*)(he->key.pointer)),
                     tempus, status);
-                he = variableNames.nextElement(pos);
+                he = variableNames.nextElement(p);
             }
         }
         variablesVector.removeAllElements();   // keeps them from getting deleted when we succeed
@@ -1157,7 +1177,7 @@ void TransliteratorParser::setVariableRange(int32_t start, int32_t end, UErrorCo
 
 /**
  * Assert that the given character is NOT within the variable range.
- * If it is, return FALSE.  This is neccesary to ensure that the
+ * If it is, return false.  This is necessary to ensure that the
  * variable range does not overlap characters used in a rule.
  */
 UBool TransliteratorParser::checkVariableRange(UChar32 ch) const {
@@ -1198,7 +1218,7 @@ static const UChar PRAGMA_NFC_RULES[] = {0x7E,0x6E,0x66,0x63,0x20,0x72,0x75,0x6C
  */
 UBool TransliteratorParser::resemblesPragma(const UnicodeString& rule, int32_t pos, int32_t limit) {
     // Must start with /use\s/i
-    return ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(TRUE, PRAGMA_USE, 4), NULL) >= 0;
+    return ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(true, PRAGMA_USE, 4), NULL) >= 0;
 }
 
 /**
@@ -1223,25 +1243,25 @@ int32_t TransliteratorParser::parsePragma(const UnicodeString& rule, int32_t pos
     // use maximum backup 16;
     // use nfd rules;
     // use nfc rules;
-    int p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(TRUE, PRAGMA_VARIABLE_RANGE, -1), array);
+    int p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(true, PRAGMA_VARIABLE_RANGE, -1), array);
     if (p >= 0) {
         setVariableRange(array[0], array[1], status);
         return p;
     }
     
-    p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(TRUE, PRAGMA_MAXIMUM_BACKUP, -1), array);
+    p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(true, PRAGMA_MAXIMUM_BACKUP, -1), array);
     if (p >= 0) {
         pragmaMaximumBackup(array[0]);
         return p;
     }
     
-    p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(TRUE, PRAGMA_NFD_RULES, -1), NULL);
+    p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(true, PRAGMA_NFD_RULES, -1), NULL);
     if (p >= 0) {
         pragmaNormalizeRules(UNORM_NFD);
         return p;
     }
     
-    p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(TRUE, PRAGMA_NFC_RULES, -1), NULL);
+    p = ICU_Utility::parsePattern(rule, pos, limit, UnicodeString(true, PRAGMA_NFC_RULES, -1), NULL);
     if (p >= 0) {
         pragmaNormalizeRules(UNORM_NFC);
         return p;
@@ -1536,6 +1556,10 @@ UChar TransliteratorParser::generateStandInFor(UnicodeFunctor* adopted, UErrorCo
         return 0;
     }
     variablesVector.addElement(adopted, status);
+    if (U_FAILURE(status)) {
+        delete adopted;
+        return 0;
+    }
     return variableNext++;
 }
 
@@ -1555,7 +1579,7 @@ UChar TransliteratorParser::getSegmentStandin(int32_t seg, UErrorCode& status) {
             return 0;
         }
         c = variableNext++;
-        // Set a placeholder in the master variables vector that will be
+        // Set a placeholder in the primary variables vector that will be
         // filled in later by setSegmentObject().  We know that we will get
         // called first because setSegmentObject() will call us.
         variablesVector.addElement((void*) NULL, status);
@@ -1575,13 +1599,17 @@ void TransliteratorParser::setSegmentObject(int32_t seg, StringMatcher* adopted,
     if (segmentObjects.size() < seg) {
         segmentObjects.setSize(seg, status);
     }
+    if (U_FAILURE(status)) {
+        return;
+    }
     int32_t index = getSegmentStandin(seg, status) - curData->variablesBase;
     if (segmentObjects.elementAt(seg-1) != NULL ||
         variablesVector.elementAt(index) != NULL) {
         // should never happen
-        status = U_INTERNAL_TRANSLITERATOR_ERROR;
+        if (U_SUCCESS(status)) {status = U_INTERNAL_TRANSLITERATOR_ERROR;}
         return;
     }
+    // Note: neither segmentObjects or variablesVector has an object deleter function.
     segmentObjects.setElementAt(adopted, seg-1);
     variablesVector.setElementAt(adopted, index);
 }
@@ -1592,7 +1620,7 @@ void TransliteratorParser::setSegmentObject(int32_t seg, StringMatcher* adopted,
  */
 UChar TransliteratorParser::getDotStandIn(UErrorCode& status) {
     if (dotStandIn == (UChar) -1) {
-        UnicodeSet* tempus = new UnicodeSet(UnicodeString(TRUE, DOT_SET, -1), status);
+        UnicodeSet* tempus = new UnicodeSet(UnicodeString(true, DOT_SET, -1), status);
         // Null pointer check.
         if (tempus == NULL) {
             status = U_MEMORY_ALLOCATION_ERROR;
@@ -1653,7 +1681,7 @@ utrans_stripRules(const UChar *source, int32_t sourceLen, UChar *target, UErrorC
     const UChar *sourceLimit = source+sourceLen;
     UChar *targetLimit = target+sourceLen;
     UChar32 c = 0;
-    UBool quoted = FALSE;
+    UBool quoted = false;
     int32_t index;
 
     uprv_memset(target, 0, sourceLen*U_SIZEOF_UCHAR);
@@ -1674,11 +1702,18 @@ utrans_stripRules(const UChar *source, int32_t sourceLen, UChar *target, UErrorC
                     target--;
                 }
                 do {
+                    if (source == sourceLimit) {
+                        c = U_SENTINEL;
+                        break;
+                    }
                     c = *(source++);
                 }
                 while (c != CR && c != LF);
+                if (c < 0) {
+                    break;
+                }
             }
-            else if (c == ESCAPE) {
+            else if (c == ESCAPE && source < sourceLimit) {
                 UChar32   c2 = *source;
                 if (c2 == CR || c2 == LF) {
                     /* A backslash at the end of a line. */
@@ -1713,7 +1748,7 @@ utrans_stripRules(const UChar *source, int32_t sourceLen, UChar *target, UErrorC
             /* ignore spaces carriage returns, and all leading spaces on the next line.
             * and line feed unless in the form \uXXXX
             */
-            quoted = FALSE;
+            quoted = false;
             while (source < sourceLimit) {
                 c = *(source);
                 if (c != CR && c != LF && c != 0x0020) {
