@@ -1,20 +1,30 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
-**********************************************************************
-*   Copyright (C) 1999, International Business Machines
-*   Corporation and others.  All Rights Reserved.
-**********************************************************************
-*   Date        Name        Description
-*   11/17/99    aliu        Creation.
-**********************************************************************
-*/
+ **********************************************************************
+ *   Copyright (C) 1999-2011, International Business Machines
+ *   Corporation and others.  All Rights Reserved.
+ **********************************************************************
+ *   Date        Name        Description
+ *   11/17/99    aliu        Creation.
+ **********************************************************************
+ */
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
+#include "unicode/unistr.h"
+#include "unicode/uniset.h"
+#include "unicode/utf16.h"
 #include "rbt_set.h"
 #include "rbt_rule.h"
-#include "unicode/unistr.h"
 #include "cmemory.h"
+#include "putilimp.h"
 
 U_CDECL_BEGIN
-static void U_EXPORT2 U_CALLCONV _deleteRule(void *rule) {
-    delete (U_NAMESPACE_QUALIFIER TransliterationRule *)rule;
+static void U_CALLCONV _deleteRule(void *rule) {
+    delete (icu::TransliterationRule *)rule;
 }
 U_CDECL_END
 
@@ -83,7 +93,7 @@ UnicodeString& _escape(const UnicodeString &source,
                        UnicodeString &target) {
     for (int32_t i = 0; i < source.length(); ) {
         UChar32 ch = source.char32At(i);
-        i += UTF_CHAR_LENGTH(ch);
+        i += U16_LENGTH(ch);
         if (ch < 0x09 || (ch > 0x0A && ch < 0x20)|| ch > 0x7E) {
             if (ch <= 0xFFFF) {
                 target += "\\u";
@@ -98,15 +108,13 @@ UnicodeString& _escape(const UnicodeString &source,
     }
     return target;
 }
-#endif
 
 inline void _debugOut(const char* msg, TransliterationRule* rule,
                       const Replaceable& theText, UTransPosition& pos) {
-#ifdef DEBUG_RBT
     UnicodeString buf(msg, "");
     if (rule) {
         UnicodeString r;
-        rule->toRule(r, TRUE);
+        rule->toRule(r, true);
         buf.append((UChar)32).append(r);
     }
     buf.append(UnicodeString(" => ", ""));
@@ -115,33 +123,36 @@ inline void _debugOut(const char* msg, TransliterationRule* rule,
     UnicodeString esc;
     _escape(buf, esc);
     CharString cbuf(esc);
-    printf("%s\n", (char*) cbuf);
-#endif
+    printf("%s\n", (const char*) cbuf);
 }
 
+#else
+#define _debugOut(msg, rule, theText, pos)
+#endif
+
 //----------------------------------------------------------------------
 // END Debugging support
 //----------------------------------------------------------------------
 
 // Fill the precontext and postcontext with the patterns of the rules
 // that are masking one another.
-static void maskingError(const U_NAMESPACE_QUALIFIER TransliterationRule& rule1,
-                         const U_NAMESPACE_QUALIFIER TransliterationRule& rule2,
+static void maskingError(const icu::TransliterationRule& rule1,
+                         const icu::TransliterationRule& rule2,
                          UParseError& parseError) {
-    U_NAMESPACE_QUALIFIER UnicodeString r;
+    icu::UnicodeString r;
     int32_t len;
 
     parseError.line = parseError.offset = -1;
     
     // for pre-context
-    rule1.toRule(r, FALSE);
+    rule1.toRule(r, false);
     len = uprv_min(r.length(), U_PARSE_CONTEXT_LEN-1);
     r.extract(0, len, parseError.preContext);
     parseError.preContext[len] = 0;   
     
     //for post-context
     r.truncate(0);
-    rule2.toRule(r, FALSE);
+    rule2.toRule(r, false);
     len = uprv_min(r.length(), U_PARSE_CONTEXT_LEN-1);
     r.extract(0, len, parseError.postContext);
     parseError.postContext[len] = 0;   
@@ -152,35 +163,41 @@ U_NAMESPACE_BEGIN
 /**
  * Construct a new empty rule set.
  */
-TransliterationRuleSet::TransliterationRuleSet(UErrorCode& status) {
-    ruleVector = new UVector(&_deleteRule, NULL, status);
-    rules = NULL;
-    maxContextLength = 0;
-    if (ruleVector == NULL) {
-        status = U_MEMORY_ALLOCATION_ERROR;
+TransliterationRuleSet::TransliterationRuleSet(UErrorCode& status) :
+        UMemory(), ruleVector(nullptr), rules(nullptr), index {}, maxContextLength(0) {
+    LocalPointer<UVector> lpRuleVector(new UVector(_deleteRule, nullptr, status), status);
+    if (U_FAILURE(status)) {
+        return;
     }
+    ruleVector = lpRuleVector.orphan();
 }
 
 /**
  * Copy constructor.
  */
 TransliterationRuleSet::TransliterationRuleSet(const TransliterationRuleSet& other) :
-    ruleVector(0),
-    rules(0),
+    UMemory(other),
+    ruleVector(nullptr),
+    rules(nullptr),
     maxContextLength(other.maxContextLength) {
 
     int32_t i, len;
     uprv_memcpy(index, other.index, sizeof(index));
     UErrorCode status = U_ZERO_ERROR;
-    ruleVector = new UVector(&_deleteRule, NULL, status);
-    if (other.ruleVector != 0 && ruleVector != 0 && U_SUCCESS(status)) {
+    LocalPointer<UVector> lpRuleVector(new UVector(_deleteRule, nullptr, status), status);
+    if (U_FAILURE(status)) {
+        return;
+    }
+    ruleVector = lpRuleVector.orphan();
+    if (other.ruleVector != nullptr && U_SUCCESS(status)) {
         len = other.ruleVector->size();
         for (i=0; i<len && U_SUCCESS(status); ++i) {
-            ruleVector->addElement(new TransliterationRule(
-              *(TransliterationRule*)other.ruleVector->elementAt(i)), status);
+            LocalPointer<TransliterationRule> tempTranslitRule(
+                new TransliterationRule(*(TransliterationRule*)other.ruleVector->elementAt(i)), status);
+            ruleVector->adoptElement(tempTranslitRule.orphan(), status);
         }
     }
-    if (other.rules != 0) {
+    if (other.rules != 0 && U_SUCCESS(status)) {
         UParseError p;
         freeze(p, status);
     }
@@ -191,7 +208,7 @@ TransliterationRuleSet::TransliterationRuleSet(const TransliterationRuleSet& oth
  */
 TransliterationRuleSet::~TransliterationRuleSet() {
     delete ruleVector; // This deletes the contained rules
-    delete[] rules;
+    uprv_free(rules);
 }
 
 void TransliterationRuleSet::setData(const TransliterationRuleData* d) {
@@ -224,18 +241,18 @@ int32_t TransliterationRuleSet::getMaximumContextLength(void) const {
  */
 void TransliterationRuleSet::addRule(TransliterationRule* adoptedRule,
                                      UErrorCode& status) {
+    LocalPointer<TransliterationRule> lpAdoptedRule(adoptedRule);
+    ruleVector->adoptElement(lpAdoptedRule.orphan(), status);
     if (U_FAILURE(status)) {
-        delete adoptedRule;
         return;
     }
-    ruleVector->addElement(adoptedRule, status);
 
     int32_t len;
     if ((len = adoptedRule->getContextLength()) > maxContextLength) {
         maxContextLength = len;
     }
 
-    delete rules;
+    uprv_free(rules);
     rules = 0;
 }
 
@@ -279,6 +296,11 @@ void TransliterationRuleSet::freeze(UParseError& parseError,UErrorCode& status)
      * Be careful not to call malloc(0).
      */
     int16_t* indexValue = (int16_t*) uprv_malloc( sizeof(int16_t) * (n > 0 ? n : 1) );
+    /* test for NULL */
+    if (indexValue == 0) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        return;
+    }
     for (j=0; j<n; ++j) {
         TransliterationRule* r = (TransliterationRule*) ruleVector->elementAt(j);
         indexValue[j] = r->getIndexValue();
@@ -294,7 +316,7 @@ void TransliterationRuleSet::freeze(UParseError& parseError,UErrorCode& status)
                 // If the indexValue is < 0, then the first key character is
                 // a set, and we must use the more time-consuming
                 // matchesIndexValue check.  In practice this happens
-                // rarely, so we seldom tread this code path.
+                // rarely, so we seldom treat this code path.
                 TransliterationRule* r = (TransliterationRule*) ruleVector->elementAt(j);
                 if (r->matchesIndexValue((uint8_t)x)) {
                     v.addElement(r, status);
@@ -304,11 +326,25 @@ void TransliterationRuleSet::freeze(UParseError& parseError,UErrorCode& status)
     }
     uprv_free(indexValue);
     index[256] = v.size();
+    if (U_FAILURE(status)) {
+        return;
+    }
 
     /* Freeze things into an array.
      */
-    delete[] rules; // Contains alias pointers
-    rules = new TransliterationRule*[v.size()];
+    uprv_free(rules); // Contains alias pointers
+
+    /* You can't do malloc(0)! */
+    if (v.size() == 0) {
+        rules = NULL;
+        return;
+    }
+    rules = (TransliterationRule **)uprv_malloc(v.size() * sizeof(TransliterationRule *));
+    /* test for NULL */
+    if (rules == 0) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        return;
+    }
     for (j=0; j<v.size(); ++j) {
         rules[j] = (TransliterationRule*) v.elementAt(j);
     }
@@ -351,14 +387,14 @@ void TransliterationRuleSet::freeze(UParseError& parseError,UErrorCode& status)
 
 /**
  * Transliterate the given text with the given UTransPosition
- * indices.  Return TRUE if the transliteration should continue
- * or FALSE if it should halt (because of a U_PARTIAL_MATCH match).
- * Note that FALSE is only ever returned if isIncremental is TRUE.
+ * indices.  Return true if the transliteration should continue
+ * or false if it should halt (because of a U_PARTIAL_MATCH match).
+ * Note that false is only ever returned if isIncremental is true.
  * @param text the text to be transliterated
  * @param pos the position indices, which will be updated
- * @param incremental if TRUE, assume new text may be inserted
- * at index.limit, and return FALSE if thre is a partial match.
- * @return TRUE unless a U_PARTIAL_MATCH has been obtained,
+ * @param incremental if true, assume new text may be inserted
+ * at index.limit, and return false if there is a partial match.
+ * @return true unless a U_PARTIAL_MATCH has been obtained,
  * indicating that transliteration should stop until more text
  * arrives.
  */
@@ -371,18 +407,18 @@ UBool TransliterationRuleSet::transliterate(Replaceable& text,
         switch (m) {
         case U_MATCH:
             _debugOut("match", rules[i], text, pos);
-            return TRUE;
+            return true;
         case U_PARTIAL_MATCH:
             _debugOut("partial match", rules[i], text, pos);
-            return FALSE;
+            return false;
         default: /* Ram: added default to make GCC happy */
             break;
         }
     }
     // No match or partial match from any rule
-    pos.start += UTF_CHAR_LENGTH(text.char32At(pos.start));
+    pos.start += U16_LENGTH(text.char32At(pos.start));
     _debugOut("no match", NULL, text, pos);
-    return TRUE;
+    return true;
 }
 
 /**
@@ -404,4 +440,27 @@ UnicodeString& TransliterationRuleSet::toRules(UnicodeString& ruleSource,
     return ruleSource;
 }
 
+/**
+ * Return the set of all characters that may be modified
+ * (getTarget=false) or emitted (getTarget=true) by this set.
+ */
+UnicodeSet& TransliterationRuleSet::getSourceTargetSet(UnicodeSet& result,
+                               UBool getTarget) const
+{
+    result.clear();
+    int32_t count = ruleVector->size();
+    for (int32_t i=0; i<count; ++i) {
+        TransliterationRule* r =
+            (TransliterationRule*) ruleVector->elementAt(i);
+        if (getTarget) {
+            r->addTargetSetTo(result);
+        } else {
+            r->addSourceSetTo(result);
+        }
+    }
+    return result;
+}
+
 U_NAMESPACE_END
+
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
