@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
-* Copyright (C) 2007-2011, International Business Machines Corporation and    *
+* Copyright (C) 2007-2013, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 */
@@ -16,11 +18,11 @@
 
 U_NAMESPACE_BEGIN
 
-typedef struct OlsonToMetaMappingEntry {
+struct OlsonToMetaMappingEntry : public UMemory {
     const UChar *mzid; // const because it's a reference to a resource bundle string.
     UDate from;
     UDate to;
-} OlsonToMetaMappingEntry;
+};
 
 class UVector;
 class TimeZone;
@@ -38,29 +40,29 @@ public:
 
     /**
      * Return the canonical id for this tzid defined by CLDR, which might be the id itself.
-     * This overload method returns a persistent const UChar*, which is guranteed to persist
-     * (a pointer to a resource).
+     * This overload method returns a persistent const UChar*, which is guaranteed to persist
+     * (a pointer to a resource). If the given system tzid is not known, U_ILLEGAL_ARGUMENT_ERROR
+     * is set in the status.
+     * @param tzid Zone ID
+     * @param status Receives the status
+     * @return The canonical ID for the input time zone ID
      */
     static const UChar* U_EXPORT2 getCanonicalCLDRID(const UnicodeString &tzid, UErrorCode& status);
 
     /*
-     * Conveninent method returning CLDR canonical ID for the given time zone
+     * Convenient method returning CLDR canonical ID for the given time zone
      */
     static const UChar* U_EXPORT2 getCanonicalCLDRID(const TimeZone& tz);
 
     /**
      * Return the canonical country code for this tzid.  If we have none, or if the time zone
-     * is not associated with a country, return null.
+     * is not associated with a country, return bogus string.
+     * @param tzid Zone ID
+     * @param country [output] Country code
+     * @param isPrimary [output] true if the zone is the primary zone for the country
+     * @return A reference to the result country
      */
-    static UnicodeString& U_EXPORT2 getCanonicalCountry(const UnicodeString &tzid, UnicodeString &canonicalCountry);
-
-    /**
-     * Return the country code if this is a 'single' time zone that can fallback to just
-     * the country, otherwise return empty string.  (Note, one must also check the locale data
-     * to see that there is a localization for the country in order to implement
-     * tr#35 appendix J step 5.)
-     */
-    static UnicodeString& U_EXPORT2 getSingleCountry(const UnicodeString &tzid, UnicodeString &country);
+    static UnicodeString& U_EXPORT2 getCanonicalCountry(const UnicodeString &tzid, UnicodeString &country, UBool *isPrimary = NULL);
 
     /**
      * Returns a CLDR metazone ID for the given Olson tzid and time.
@@ -87,10 +89,34 @@ public:
      */
     static const UChar* U_EXPORT2 findMetaZoneID(const UnicodeString& mzid);
 
+    /**
+     * Creates a custom zone for the offset
+     * @param offset GMT offset in milliseconds
+     * @return A custom TimeZone for the offset with normalized time zone id
+     */
+    static TimeZone* createCustomTimeZone(int32_t offset);
+
+    /**
+     * Returns the time zone's short ID (null terminated) for the zone.
+     * For example, "uslax" for zone "America/Los_Angeles".
+     * @param tz the time zone
+     * @return the short ID of the time zone, or null if the short ID is not available.
+     */
+    static const UChar* U_EXPORT2 getShortID(const TimeZone& tz);
+
+    /**
+     * Returns the time zone's short ID (null terminated) for the zone ID.
+     * For example, "uslax" for zone ID "America/Los_Angeles".
+     * @param tz the time zone ID
+     * @return the short ID of the time zone ID, or null if the short ID is not available.
+     */
+    static const UChar* U_EXPORT2 getShortID(const UnicodeString& id);
+
 private:
-    ZoneMeta(); // Prevent construction.
+    ZoneMeta() = delete; // Prevent construction.
     static UVector* createMetazoneMappings(const UnicodeString &tzid);
-    static void initAvailableMetaZoneIDs();
+    static UnicodeString& formatCustomID(uint8_t hour, uint8_t min, uint8_t sec, UBool negative, UnicodeString& id);
+    static const UChar* getShortIDFromCanonical(const UChar* canonicalID);
 };
 
 U_NAMESPACE_END
