@@ -21,6 +21,8 @@
 
 #if !UCONFIG_NO_FORMATTING
 
+#include <stdbool.h>
+
 #include "unicode/uloc.h"
 #include "unicode/udat.h"
 #include "unicode/ucal.h"
@@ -69,13 +71,13 @@ void Test4029195()
         return;
     }
     resultlength=0;
-    resultlengthneeded=udat_toPattern(df, TRUE, NULL, resultlength, &status);
+    resultlengthneeded=udat_toPattern(df, true, NULL, resultlength, &status);
     if(status==U_BUFFER_OVERFLOW_ERROR)
     {
         status=U_ZERO_ERROR;
         resultlength=resultlengthneeded + 1;
         pat=(UChar*)malloc(sizeof(UChar) * resultlength);
-        udat_toPattern(df, TRUE, pat, resultlength, &status);
+        udat_toPattern(df, true, pat, resultlength, &status);
     }
     
     log_verbose("pattern: %s\n", austrdup(pat));
@@ -85,13 +87,13 @@ void Test4029195()
     if(fmdt) {
       log_verbose("today: %s\n", austrdup(fmdt));
     } else {
-      log_data_err("ERROR: couldn't format, exitting test");
+      log_data_err("ERROR: couldn't format, exiting test");
       return;
     }
     
     temp=(UChar*)malloc(sizeof(UChar) * 10);
     u_uastrcpy(temp, "M yyyy dd");
-    udat_applyPattern(df, TRUE, temp, u_strlen(temp));
+    udat_applyPattern(df, true, temp, u_strlen(temp));
     
     todayS =myFormatit(df, today);
     log_verbose("After the pattern is applied\n today: %s\n", austrdup(todayS) );
@@ -166,6 +168,7 @@ void Test4056591()
     if(U_FAILURE(status))
     {
         log_data_err("FAIL: error in creating the dateformat using u_openPattern(): %s - (Are you missing data?)\n", myErrorName(status));
+        ucal_close(cal);
         return;
     }
     start = 1800;
@@ -247,13 +250,13 @@ void aux917( UDateFormat *fmt, UChar* str)
     UDate d1=1000000000.0;
    
     resultlength=0;
-    resultlengthneeded=udat_toPattern(fmt, TRUE, NULL, resultlength, &status);
+    resultlengthneeded=udat_toPattern(fmt, true, NULL, resultlength, &status);
     if(status==U_BUFFER_OVERFLOW_ERROR)
     {
         status=U_ZERO_ERROR;
         resultlength=resultlengthneeded + 1;
         pat=(UChar*)malloc(sizeof(UChar) * (resultlength));
-        udat_toPattern(fmt, TRUE, pat, resultlength, &status);
+        udat_toPattern(fmt, true, pat, resultlength, &status);
     }
     if(U_FAILURE(status)){
         log_err("failure in retrieving the pattern: %s\n", myErrorName(status));
@@ -340,17 +343,17 @@ void Test4061287()
 
     pos=0;
     
-    udat_setLenient(df, FALSE);
+    udat_setLenient(df, false);
     ok=udat_isLenient(df);
-    if(ok==TRUE)
+    if(ok==true)
         log_err("setLenient nor working\n");
-    ok = FALSE;
+    ok = false;
     myDate = udat_parse(df, dateString, u_strlen(dateString), &pos, &status);
     (void)myDate;   /* Suppress set but not used warning. */
     if(U_FAILURE(status))
-        ok = TRUE;
-    if(ok!=TRUE) 
-        log_err("Fail: Lenient not working: does lenient parsing in spite of setting Leninent as FALSE ");
+        ok = true;
+    if(ok!=true) 
+        log_err("Fail: Lenient not working: does lenient parsing in spite of setting Lenient as false ");
 
     udat_close(df);
     
@@ -395,7 +398,7 @@ void Test4073003()
         return;
     }
     u_uastrcpy(temp, "m/D/yy");
-    udat_applyPattern(fmt, FALSE, temp, u_strlen(temp));
+    udat_applyPattern(fmt, false, temp, u_strlen(temp));
 
     for(i= 0; i < 4; i+=2) {
         status=U_ZERO_ERROR;
@@ -423,7 +426,7 @@ void Test4073003()
         result =myFormatit(fmt, d);
         result2 =myFormatit(fmt, dd);
         if(!result || !result2) {
-            log_data_err("Fail: could not format - exitting test\n");
+            log_data_err("Fail: could not format - exiting test\n");
             return;
         }
         if (u_strcmp(result, result2)!=0){
@@ -473,11 +476,10 @@ void Test4162071()
 void Test714(void)
 {
     UDate d=978103543000.0;
-    UChar temp[20];
     UErrorCode status = U_ZERO_ERROR;
     UDateFormat *fmt;
     UChar *result;
-    const char* expect =  "7:25:43 AM";
+    const UChar* expect =  u"7:25:43\u202FAM";
     
     ctest_setTimeZone(NULL, &status);
 
@@ -490,15 +492,14 @@ void Test714(void)
     }
     result =myFormatit(fmt, d);
     if(!result) {
-      log_data_err("Fail: could not format - exitting test\n");
+      log_data_err("Fail: could not format - exiting test\n");
       return;
     }
-    u_uastrcpy(temp, expect);
-    if (u_strcmp(result, temp)!=0){
-      log_err("Fail: %s != %s\n", austrdup(result), expect);
+    if (u_strcmp(result, expect)!=0){
+      log_err("Fail: %s != %s\n", austrdup(result), austrdup(expect));
     }
     else{
-      log_verbose("Ok: %s == %s\n", austrdup(result), expect );
+      log_verbose("Ok: %s == %s\n", austrdup(result), austrdup(expect));
     }
         
     udat_close(fmt);
@@ -575,7 +576,7 @@ void Test_GEec(void)
             int32_t dmyGnTextLen;
             UDate   dateResult;
 
-            udat_applyPattern(dtfmt, FALSE, patTextPtr->pattern, -1);
+            udat_applyPattern(dtfmt, false, patTextPtr->pattern, -1);
             dmyGnTextLen = udat_format(dtfmt, july022008, dmyGnText, DATE_TEXT_MAX_CHARS, NULL, &status);
             (void)dmyGnTextLen;   /* Suppress set but not used warning. */ 
             if ( U_FAILURE(status) ) {
