@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1998-2011, International Business Machines Corporation and
+ * Copyright (c) 1998-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /*
@@ -23,8 +25,29 @@
 #include "putilimp.h"
 #include "toolutil.h"
 #include "uinvchar.h"
+#include <stdbool.h>
 #include <stdio.h>
+#if U_PLATFORM_USES_ONLY_WIN32_API 
+#include "wintz.h"
+#endif
 
+/* See the comments on U_SIGNED_RIGHT_SHIFT_IS_ARITHMETIC. */
+static void TestSignedRightShiftIsArithmetic(void) {
+    int32_t x=0xfff5fff3;
+    int32_t m=-1;
+    int32_t x4=x>>4;
+    int32_t m1=m>>1;
+    UBool signedRightShiftIsArithmetic= (x4==(int32_t)0xffff5fff && m1==-1);
+    if(signedRightShiftIsArithmetic==U_SIGNED_RIGHT_SHIFT_IS_ARITHMETIC) {
+        log_info("signed right shift is Arithmetic Shift Right: %d\n",
+                 signedRightShiftIsArithmetic);
+    } else {
+        log_err("error: unexpected signed right shift is Arithmetic Shift Right: %d\n"
+                "       You need to change the value of U_SIGNED_RIGHT_SHIFT_IS_ARITHMETIC "
+                "for your platform.\n",
+                signedRightShiftIsArithmetic);
+    }
+}
 
 static UBool compareWithNAN(double x, double y);
 static void doAssert(double expect, double got, const char *message);
@@ -34,7 +57,7 @@ static void TestPUtilAPI(void){
     double  n1=0.0, y1=0.0, expn1, expy1;
     double  value1 = 0.021;
     char *str=0;
-    UBool isTrue=FALSE;
+    UBool isTrue=false;
 
     log_verbose("Testing the API uprv_modf()\n");
     y1 = uprv_modf(value1, &n1);
@@ -95,20 +118,34 @@ static void TestPUtilAPI(void){
 
     log_verbose("Testing the API uprv_isNegativeInfinity()\n");
     isTrue=uprv_isNegativeInfinity(uprv_getInfinity() * -1);
-    if(isTrue != TRUE){
+    if(isTrue != true){
         log_err("ERROR: uprv_isNegativeInfinity failed.\n");
     }
     log_verbose("Testing the API uprv_isPositiveInfinity()\n");
     isTrue=uprv_isPositiveInfinity(uprv_getInfinity());
-    if(isTrue != TRUE){
+    if(isTrue != true){
         log_err("ERROR: uprv_isPositiveInfinity failed.\n");
     }
     log_verbose("Testing the API uprv_isInfinite()\n");
     isTrue=uprv_isInfinite(uprv_getInfinity());
-    if(isTrue != TRUE){
+    if(isTrue != true){
         log_err("ERROR: uprv_isInfinite failed.\n");
     }
 
+    log_verbose("Testing the APIs uprv_add32_overflow and uprv_mul32_overflow\n");
+    int32_t overflow_result;
+    doAssert(false, uprv_add32_overflow(INT32_MAX - 2, 1, &overflow_result), "should not overflow");
+    doAssert(INT32_MAX - 1, overflow_result, "should equal INT32_MAX - 1");
+    doAssert(false, uprv_add32_overflow(INT32_MAX - 2, 2, &overflow_result), "should not overflow");
+    doAssert(INT32_MAX, overflow_result, "should equal exactly INT32_MAX");
+    doAssert(true, uprv_add32_overflow(INT32_MAX - 2, 3, &overflow_result), "should overflow");
+    doAssert(false, uprv_mul32_overflow(INT32_MAX / 5, 4, &overflow_result), "should not overflow");
+    doAssert(INT32_MAX / 5 * 4, overflow_result, "should equal INT32_MAX / 5 * 4");
+    doAssert(true, uprv_mul32_overflow(INT32_MAX / 5, 6, &overflow_result), "should overflow");
+    // Test on negative numbers:
+    doAssert(false, uprv_add32_overflow(-3, -2, &overflow_result), "should not overflow");
+    doAssert(-5, overflow_result, "should equal -5");
+
 #if 0
     log_verbose("Testing the API uprv_digitsAfterDecimal()....\n");
     doAssert(uprv_digitsAfterDecimal(value1), 3, "uprv_digitsAfterDecimal() failed.");
@@ -181,8 +218,19 @@ static void TestPUtilAPI(void){
             log_info("Note: t_timezone offset of %ld (for %s : %s) is not a multiple of 30min.", tzoffset, uprv_tzname(0), uprv_tzname(1));
         }
         /*tzoffset=uprv_getUTCtime();*/
-
     }
+
+#if U_PLATFORM_USES_ONLY_WIN32_API 
+    log_verbose("Testing uprv_detectWindowsTimeZone() ....\n");
+    {
+        const char* timezone = uprv_detectWindowsTimeZone();
+        if (timezone == NULL) {
+            log_err("ERROR: uprv_detectWindowsTimeZone failed (returned NULL).\n");
+        } else {
+            log_verbose("Detected TimeZone = %s\n", timezone);
+        }   
+    }
+#endif
 }
 
 static void TestVersion(void)
@@ -192,7 +240,6 @@ static void TestVersion(void)
     char versionString[17]; /* xxx.xxx.xxx.xxx\0 */
     UChar versionUString[] = { 0x0031, 0x002E, 0x0030, 0x002E,
                                0x0032, 0x002E, 0x0038, 0x0000 }; /* 1.0.2.8 */
-    UBool isModified = FALSE;
     UVersionInfo version;
     UErrorCode status = U_ZERO_ERROR;
 
@@ -280,11 +327,6 @@ static void TestVersion(void)
     u_getDataVersion(version, &status);
     if (U_FAILURE(status)) {
         log_data_err("ERROR: Unable to get data version. %s\n", u_errorName(status));
-    } else {
-        u_isDataOlder(version, &isModified, &status);
-        if (U_FAILURE(status)) {
-            log_err("ERROR: Unable to compare data version. %s\n", u_errorName(status));
-        }
     }
 }
 
@@ -396,14 +438,14 @@ static UBool compareWithNAN(double x, double y)
 {
   if( uprv_isNaN(x) || uprv_isNaN(y) ) {
     if(!uprv_isNaN(x) || !uprv_isNaN(y) ) {
-      return FALSE;
+      return false;
     }
   }
   else if (y != x) { /* no NaN's involved */
-    return FALSE;
+    return false;
   }
 
-  return TRUE;
+  return true;
 }
 
 static void doAssert(double got, double expect, const char *message)
@@ -554,7 +596,7 @@ static void toolutil_findBasename(void)
       ""
     },
   };
-  int32_t count=(sizeof(testCases)/sizeof(testCases[0]));
+  int32_t count=UPRV_LENGTHOF(testCases);
   int32_t i;
 
 
@@ -643,26 +685,14 @@ static void toolutil_findDirname(void)
       U_BUFFER_OVERFLOW_ERROR,
       NULL
     },
-    {
-      "pkgdata",
-      1,
-      U_BUFFER_OVERFLOW_ERROR,
-      NULL
-    },
     {
       "pkgdata",
       2,
       U_ZERO_ERROR,
-      "."
-    },
-    {
-      "pkgdata",
-      20,
-      U_ZERO_ERROR,
-      "."
+      ""
     }
   };
-  int32_t count=(sizeof(testCases)/sizeof(testCases[0]));
+  int32_t count=UPRV_LENGTHOF(testCases);
 
   log_verbose("Testing findDirname()\n");
   for(i=0;i<count;i++) {
@@ -691,6 +721,7 @@ static void toolutil_findDirname(void)
 static void addToolUtilTests(TestNode** root) {
     addTest(root, &toolutil_findBasename,       "putiltst/toolutil/findBasename");
     addTest(root, &toolutil_findDirname,       "putiltst/toolutil/findDirname");
+    addTest(root, &TestSignedRightShiftIsArithmetic, "putiltst/toolutil/TestSignedRightShiftIsArithmetic");
   /*
     Not yet tested:
 
