@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1998-2016, International Business Machines Corporation and
@@ -29,8 +31,9 @@
 #include "cintltst.h"
 #include "ubrkimpl.h"
 #include "toolutil.h" /* for uprv_fileExists() */
-#include <stdlib.h>
+#include <stdbool.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 /* includes for TestSwapData() */
 #include "udataswp.h"
@@ -556,6 +559,7 @@ static UBool U_CALLCONV
 isAcceptable1(void *context,
              const char *type, const char *name,
              const UDataInfo *pInfo) {
+    (void)context; // suppress compiler warnings about unused variable
 
     if( pInfo->size>=20 &&
         pInfo->isBigEndian==U_IS_BIG_ENDIAN &&
@@ -566,10 +570,10 @@ isAcceptable1(void *context,
         pInfo->dataFormat[3]==0x6c &&
         pInfo->formatVersion[0]==3 )
     {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable1()\n", name, type);
-        return TRUE;
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable1()\n", name, type);
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable1():-\n"
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable1():-\n"
             "\tsize              = %d\n"
             "\tisBigEndian       = %d\n"
             "\tcharsetFamily     = %d\n"
@@ -579,8 +583,8 @@ isAcceptable1(void *context,
             name, type, pInfo->size,  pInfo->isBigEndian, pInfo->charsetFamily, pInfo->formatVersion[0], 
             pInfo->dataVersion[0], pInfo->dataFormat[0], pInfo->dataFormat[1], pInfo->dataFormat[2], 
             pInfo->dataFormat[3]);  
-        log_verbose("Call another verifing function to accept the data\n");
-        return FALSE;
+        log_verbose("Call another verifying function to accept the data\n");
+        return false;
     }
 }
 
@@ -588,6 +592,7 @@ static UBool U_CALLCONV
 isAcceptable2(void *context, 
              const char *type, const char *name,
       const UDataInfo *pInfo){
+    (void)context; // suppress compiler warnings about unused variable
     UVersionInfo unicodeVersion;
 
     u_getUnicodeVersion(unicodeVersion);
@@ -602,12 +607,12 @@ isAcceptable2(void *context,
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==unicodeVersion[0] )
     {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable2()\n", name, type);
-        return TRUE;
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable2()\n", name, type);
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable2()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable2()\n", name, type);
 
-        return FALSE;
+        return false;
     }
 
 
@@ -616,6 +621,7 @@ static UBool U_CALLCONV
 isAcceptable3(void *context, 
              const char *type, const char *name,
              const UDataInfo *pInfo){
+    (void)context; // suppress compiler warnings about unused variable
 
     if( pInfo->size>=20 &&
         pInfo->isBigEndian==U_IS_BIG_ENDIAN &&
@@ -626,12 +632,12 @@ isAcceptable3(void *context,
         pInfo->dataFormat[3]==0x74 &&
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==1   ) {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable3()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable3()\n", name, type);
 
-        return TRUE;
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable3()\n", name, type);
-        return FALSE;
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable3()\n", name, type);
+        return false;
     }
 
 
@@ -726,12 +732,12 @@ isAcceptable(void *context,
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==1   &&
         *((int*)context) == 2 ) {
-        log_verbose("The data from\"%s.%s\" IS acceptable using the verifing function isAcceptable()\n", name, type);
+        log_verbose("The data from\"%s.%s\" IS acceptable using the verifying function isAcceptable()\n", name, type);
 
-        return TRUE;
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable()\n", name, type);
-        return FALSE;
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable()\n", name, type);
+        return false;
     }
 }
 
@@ -1231,7 +1237,8 @@ static const struct {
 
     {0x31, 0x31, 0x31, 0x31},     /* dataFormat="1111" */
     {0, 0, 0, 0},                 /* formatVersion */
-    {0, 0, 0, 0}}                 /* dataVersion */
+    {0, 0, 0, 0}},                /* dataVersion */
+    0
 };
 #endif
 
@@ -1347,7 +1354,7 @@ static const struct {
 
 #if !UCONFIG_NO_BREAK_ITERATION
     {"char",                     "brk", ubrk_swap},
-    {"thaidict",                 "dict",udict_swap},
+    {"laodict",                  "dict",udict_swap},
 #endif
 
 #if 0
@@ -1393,6 +1400,7 @@ static const struct {
 
 static void U_CALLCONV
 printError(void *context, const char *fmt, va_list args) {
+    (void)context; // suppress compiler warnings about unused variable
     vlog_info("[swap] ", fmt, args);
     log_err("\n");  /* Register error */
 }
@@ -1850,7 +1858,7 @@ static void TestTZDataDir(void) {
     //     whatever/.../testdata/out/testdata
     // The test data puts an old (2014a) version of the time zone data there.
 
-    // Switch ICU to the testdata version of zoneinfo64.res, which is verison 2014a.
+    // Switch ICU to the testdata version of zoneinfo64.res, which is version 2014a.
     ctest_resetICU();
     u_setTimeZoneFilesDirectory(testDataPath, &status);
     tzDataVersion = ucal_getTZDataVersion(&status);
