@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /***********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2014, International Business Machines Corporation
+ * Copyright (c) 1997-2015, International Business Machines Corporation
  * and others. All Rights Reserved.
  ***********************************************************************/
  
@@ -22,7 +24,7 @@
 // class DateFormatRoundTripTest
 // *****************************************************************************
 
-// Useful for turning up subtle bugs: Change the following to TRUE, recompile,
+// Useful for turning up subtle bugs: Change the following to true, recompile,
 // and run while at lunch.
 // Warning -- makes test run infinite loop!!!
 #ifndef INFINITE
@@ -65,10 +67,10 @@ DateFormatRoundTripTest::failure(UErrorCode status, const char* msg)
 {
     if(U_FAILURE(status)) {
         errln(UnicodeString("FAIL: ") + msg + " failed, error " + u_errorName(status));
-        return TRUE;
+        return true;
     }
 
-    return FALSE;
+    return false;
 }
 
 UBool 
@@ -78,10 +80,10 @@ DateFormatRoundTripTest::failure(UErrorCode status, const char* msg, const Unico
         UnicodeString escaped;
         escape(str,escaped);
         errln(UnicodeString("FAIL: ") + msg + " failed, error " + u_errorName(status) + ", str=" + escaped);
-        return TRUE;
+        return true;
     }
 
-    return FALSE;
+    return false;
 }
 
 void DateFormatRoundTripTest::TestCentury()
@@ -218,14 +220,14 @@ void DateFormatRoundTripTest::test(const Locale& loc)
     UBool TEST_TABLE [24];//= new boolean[24];
     int32_t i = 0;
     for(i = 0; i < 24; ++i) 
-        TEST_TABLE[i] = TRUE;
+        TEST_TABLE[i] = true;
 
     // If we have some sparseness, implement it here.  Sparseness decreases
     // test time by eliminating some tests, up to 23.
     for(i = 0; i < SPARSENESS; ) {
         int random = (int)(randFraction() * 24);
         if (random >= 0 && random < 24 && TEST_TABLE[i]) {
-            TEST_TABLE[i] = FALSE;
+            TEST_TABLE[i] = false;
             ++i;
         }
     }
@@ -247,12 +249,12 @@ void DateFormatRoundTripTest::test(const Locale& loc)
     
     for(style = DateFormat::FULL; style <= DateFormat::SHORT; ++style) {
         if (TEST_TABLE[itable++]) {
-          logln("Testing style " + UnicodeString(styleName((DateFormat::EStyle)style)));
+            logln("Testing style " + UnicodeString(styleName((DateFormat::EStyle)style)));
             DateFormat *df = DateFormat::createTimeInstance((DateFormat::EStyle)style, loc);
             if(df == NULL) {
               errln(UnicodeString("Could not DF::createTimeInstance ") + UnicodeString(styleName((DateFormat::EStyle)style)) + " Locale: " + loc.getDisplayName(temp));
             } else {
-              test(df, loc, TRUE);
+              test(df, loc, true);
               delete df;
             }
         }
@@ -282,7 +284,7 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
         return;
     }
     
-    UBool isGregorian = FALSE;
+    UBool isGregorian = false;
     UErrorCode minStatus = U_ZERO_ERROR;
     if(fmt->getCalendar() == NULL) {
       errln((UnicodeString)"DateFormatRoundTripTest::test, DateFormat getCalendar() returns null for " + origLocale.getName());
@@ -320,7 +322,7 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
             UDate *d                = new UDate    [DEPTH];
             UnicodeString *s    = new UnicodeString[DEPTH];
 
-            if(isGregorian == TRUE) {
+            if(isGregorian == true) {
               d[0] = generateDate();
             } else {
               d[0] = generateDate(minDate);
@@ -338,8 +340,13 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
             for(loop = 0; loop < DEPTH; ++loop) {
                 if (loop > 0)  {
                     d[loop] = fmt->parse(s[loop-1], status);
-                    failure(status, "fmt->parse", s[loop-1]+" in locale: " + origLocale.getName() + " with pattern: " + pat);
-                    status = U_ZERO_ERROR; /* any error would have been reported */
+                    if(U_FAILURE(status)) {
+                        ParsePosition ppos;
+                        (void)fmt->parse(s[loop-1], ppos);
+                        failure(status, "fmt->parse", s[loop-1]+" in locale: " + origLocale.getName() +
+                                " with pattern: " + pat + " has errIndex: " + ppos.getErrorIndex());
+                        status = U_ZERO_ERROR; /* any error would have been reported */
+                    }
                 }
 
                 s[loop] = fmt->format(d[loop], s[loop]);
@@ -388,7 +395,7 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
                 // Time-only pattern with zone information and a starting date in PST.
                 if(timeOnly && hasZoneDisplayName) {
                     int32_t startRaw, startDst;
-                    fmt->getTimeZone().getOffset(d[0], FALSE, startRaw, startDst, status);
+                    fmt->getTimeZone().getOffset(d[0], false, startRaw, startDst, status);
                     failure(status, "TimeZone::getOffset");
                     // if the start offset is greater than the offset on Jan 1, 1970
                     // in PST, then need one more round trip.  There are two cases
@@ -436,6 +443,19 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
                 else if (hasZoneDisplayName && d[0] < 0) {
                     maxSmatch = 2;
                 }
+                else if (timeOnly && !isGregorian && hasZoneDisplayName && maxSmatch == 1) {
+                    int32_t startRaw, startDst;
+                    fmt->getTimeZone().getOffset(d[1], false, startRaw, startDst, status);
+                    failure(status, "TimeZone::getOffset");
+                    // If the calendar type is not Gregorian and the pattern is time only,
+                    // the calendar implementation may use a date before 1970 as day 0.
+                    // In this case, time zone offset of the default year might be
+                    // different from the one at 1970-01-01 in PST and string match requires
+                    // one more iteration.
+                    if (startRaw + startDst != -28800000) {
+                        maxSmatch = 2;
+                    }
+                }
             }
 
             /*
@@ -450,7 +470,14 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
               } else if(!strcmp(type,"hebrew")) {
                   maxSmatch = 3;
                   maxDmatch = 3;
-                }
+              } else if (timeOnly && uprv_strcmp(origLocale.getName(),"ar_JO@calendar=islamic-civil")==0 &&
+                      logKnownIssue("21049", "ar_JO@calendar=islamic-civil timeOnly roundtrip converges too slowly")) {
+                  // For some reason, for time-only tests, ar_JO@calendar=islamic-civil is no
+                  // longer converging to a match as fast as expected above. Investigate with
+                  // ICU-21022, but meanwhile allow more cycles for convergence.
+                  maxSmatch = 2;
+                  maxDmatch = 3;
+              }
             }
 
             // Use @v to see verbose results on successful cases
@@ -458,7 +485,9 @@ void DateFormatRoundTripTest::test(DateFormat *fmt, const Locale &origLocale, UB
             if (optionv || fail) {
                 if (fail) {
                     errln(UnicodeString("\nFAIL: Pattern: ") + pat +
-                          " in Locale: " + origLocale.getName());
+                          " in Locale: " + origLocale.getName() +
+                          "\nget dmatch: " + dmatch + " (expected max " + maxDmatch +
+                          "), smatch: " + smatch + " (expected max " + maxSmatch + ")");
                 } else {
                     errln(UnicodeString("\nOk: Pattern: ") + pat +
                           " in Locale: " + origLocale.getName());
@@ -519,13 +548,16 @@ UnicodeString& DateFormatRoundTripTest::escape(const UnicodeString& src, Unicode
 {
     dst.remove();
     for (int32_t i = 0; i < src.length(); ++i) {
-        UChar c = src[i];
-        if(c < 0x0080) 
+        UChar32 c = src.char32At(i);
+        if (c >= 0x10000) {
+            ++i;
+        }
+        if (c < 0x0080) {
             dst += c;
-        else {
+        } else {
             dst += UnicodeString("[");
-            char buf [8];
-            sprintf(buf, "%#x", c);
+            char buf [12];
+            sprintf(buf, "%#04x", c);
             dst += UnicodeString(buf);
             dst += UnicodeString("]");
         }
