@@ -17,11 +17,15 @@
 #include "intltest.h"
 #include "itformat.h"
 
+U_NAMESPACE_BEGIN
+class FormattedDateInterval;
+U_NAMESPACE_END
+
 /**
  * Test basic functionality of various API functions
  **/
 class DateIntervalFormatTest: public IntlTestWithFieldPosition {
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ) override;
 
 public:
     /**
@@ -34,11 +38,24 @@ public:
      */
     void testFormat();
 
+    
+    /**
+     * Test handling of hour and day period metacharacters
+     */
+    void testHourMetacharacters();
+
+    void testFormatMillisecond();
+
     /**
      * Test formatting using user defined DateIntervalInfo
      */
     void testFormatUserDII();
 
+    /*
+     * Test format using UDisplayContext
+     */
+    void testContext();
+
     /**
      * Test for no unwanted side effects when setting
      * interval patterns.
@@ -67,6 +84,13 @@ public:
     void testFormattedDateInterval();
     void testCreateInstanceForAllLocales();
 
+    void testTicket20707();
+    void testTicket21222GregorianEraDiff();
+    void testTicket21222ROCEraDiff();
+    void testTicket21222JapaneseEraDiff();
+    
+    void testTicket21939();
+
 private:
     /**
      * Test formatting against expected result
@@ -84,6 +108,19 @@ private:
      */
     void stress(const char** data, int32_t data_length, const Locale& loc,
                 const char* locName);
+
+    void getCategoryAndField(
+        const FormattedDateInterval& formatted,
+        std::vector<int32_t>& categories,
+        std::vector<int32_t>& fields,
+        IcuTestErrorCode& status);
+
+    void verifyCategoryAndField(
+        const FormattedDateInterval& formatted,
+        const std::vector<int32_t>& categories,
+        const std::vector<int32_t>& fields,
+        IcuTestErrorCode& status);
+
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
