@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  *
@@ -6,7 +8,7 @@
  *
  *******************************************************************************
  *   file name:  idnaref.cpp
- *   encoding:   US-ASCII
+ *   encoding:   UTF-8
  *   tab size:   8 (not used)
  *   indentation:4
  *
@@ -45,15 +47,15 @@ static const UChar ACE_PREFIX[] ={ 0x0078,0x006E,0x002d,0x002d } ;
 
 inline static UBool
 startsWithPrefix(const UChar* src , int32_t srcLength){
-    UBool startsWithPrefix = TRUE;
+    UBool startsWithPrefix = true;
 
     if(srcLength < ACE_PREFIX_LENGTH){
-        return FALSE;
+        return false;
     }
 
     for(int8_t i=0; i< ACE_PREFIX_LENGTH; i++){
         if(u_tolower(src[i]) != ACE_PREFIX[i]){
-            startsWithPrefix = FALSE;
+            startsWithPrefix = false;
         }
     }
     return startsWithPrefix;
@@ -196,6 +198,17 @@ CLEANUP:
     return b2Len;
 }
 
+
+static NamePrepTransform* getInstance(UErrorCode& status){
+    TestIDNA *thisTest = dynamic_cast<TestIDNA *>(IntlTest::gTest);
+    if (thisTest == nullptr && U_SUCCESS(status)) {
+        status = U_INTERNAL_PROGRAM_ERROR;
+    }
+    if (U_FAILURE(status)) return nullptr;
+    return thisTest->getInstance(status);
+}
+
+
 static int32_t convertFromPuny(  const UChar* src, int32_t srcLength,
                                  UChar* dest, int32_t destCapacity,
                                  UErrorCode& status){
@@ -269,9 +282,9 @@ idnaref_toASCII(const UChar* src, int32_t srcLength,
     UBool* caseFlags = NULL;
 
     // assume the source contains all ascii codepoints
-    UBool srcIsASCII  = TRUE;
+    UBool srcIsASCII  = true;
     // assume the source contains all LDH codepoints
-    UBool srcIsLDH = TRUE;
+    UBool srcIsLDH = true;
     int32_t j=0;
 
     if(srcLength == -1){
@@ -281,12 +294,12 @@ idnaref_toASCII(const UChar* src, int32_t srcLength,
     // step 1
     for( j=0;j<srcLength;j++){
         if(src[j] > 0x7F){
-            srcIsASCII = FALSE;
+            srcIsASCII = false;
         }
         b1[b1Len++] = src[j];
     }
 
-    NamePrepTransform* prep = TestIDNA::getInstance(*status);
+    NamePrepTransform* prep = getInstance(*status);
     if(U_FAILURE(*status)){
         goto CLEANUP;
     }
@@ -319,19 +332,19 @@ idnaref_toASCII(const UChar* src, int32_t srcLength,
         goto CLEANUP;
     }
 
-    srcIsASCII = TRUE;
+    srcIsASCII = true;
     // step 3 & 4
     for( j=0;j<b1Len;j++){
         if(b1[j] > 0x7F){// check if output of usprep_prepare is all ASCII
-            srcIsASCII = FALSE;
-        }else if(prep->isLDHChar(b1[j])==FALSE){  // if the char is in ASCII range verify that it is an LDH character{
-            srcIsLDH = FALSE;
+            srcIsASCII = false;
+        }else if(prep->isLDHChar(b1[j])==false){  // if the char is in ASCII range verify that it is an LDH character{
+            srcIsLDH = false;
         }
     }
 
-    if(useSTD3ASCIIRules == TRUE){
+    if(useSTD3ASCIIRules == true){
         // verify 3a and 3b
-        if( srcIsLDH == FALSE /* source contains some non-LDH characters */
+        if( srcIsLDH == false /* source contains some non-LDH characters */
             || b1[0] ==  HYPHEN || b1[b1Len-1] == HYPHEN){
             *status = U_IDNA_STD3_ASCII_RULES_ERROR;
             goto CLEANUP;
@@ -339,7 +352,7 @@ idnaref_toASCII(const UChar* src, int32_t srcLength,
     }
     if(srcIsASCII){
         if(b1Len <= destCapacity){
-            uprv_memmove(dest, b1, b1Len * U_SIZEOF_UCHAR);
+            u_memmove(dest, b1, b1Len);
             reqLength = b1Len;
         }else{
             reqLength = b1Len;
@@ -380,9 +393,9 @@ idnaref_toASCII(const UChar* src, int32_t srcLength,
                 goto CLEANUP;
             }
             //Step 7: prepend the ACE prefix
-            uprv_memcpy(dest,ACE_PREFIX,ACE_PREFIX_LENGTH * U_SIZEOF_UCHAR);
+            u_memcpy(dest, ACE_PREFIX, ACE_PREFIX_LENGTH);
             //Step 6: copy the contents in b2 into dest
-            uprv_memcpy(dest+ACE_PREFIX_LENGTH, b2, b2Len * U_SIZEOF_UCHAR);
+            u_memcpy(dest+ACE_PREFIX_LENGTH, b2, b2Len);
 
         }else{
             *status = U_IDNA_ACE_PREFIX_ERROR;
@@ -437,7 +450,7 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
             reqLength=0;
 //    UParseError parseError;
 
-    NamePrepTransform* prep = TestIDNA::getInstance(*status);
+    NamePrepTransform* prep = getInstance(*status);
     b1Len = 0;
     UBool* caseFlags = NULL;
 
@@ -445,8 +458,8 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
     UBool allowUnassigned   = (UBool)((options & IDNAREF_ALLOW_UNASSIGNED) != 0);
     UBool useSTD3ASCIIRules = (UBool)((options & IDNAREF_USE_STD3_RULES) != 0);
 
-    UBool srcIsASCII = TRUE;
-    UBool srcIsLDH = TRUE;
+    UBool srcIsASCII = true;
+    UBool srcIsLDH = true;
     int32_t failPos =0;
 
     if(U_FAILURE(*status)){
@@ -457,12 +470,12 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
         srcLength = 0;
         for(;src[srcLength]!=0;){
             if(src[srcLength]> 0x7f){
-                srcIsASCII = FALSE;
-            }if(prep->isLDHChar(src[srcLength])==FALSE){
+                srcIsASCII = false;
+            }if(prep->isLDHChar(src[srcLength])==false){
                 // here we do not assemble surrogates
                 // since we know that LDH code points
                 // are in the ASCII range only
-                srcIsLDH = FALSE;
+                srcIsLDH = false;
                 failPos = srcLength;
             }
             srcLength++;
@@ -470,18 +483,18 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
     }else{
         for(int32_t j=0; j<srcLength; j++){
             if(src[j]> 0x7f){
-                srcIsASCII = FALSE;
-            }else if(prep->isLDHChar(src[j])==FALSE){
+                srcIsASCII = false;
+            }else if(prep->isLDHChar(src[j])==false){
                 // here we do not assemble surrogates
                 // since we know that LDH code points
                 // are in the ASCII range only
-                srcIsLDH = FALSE;
+                srcIsLDH = false;
                 failPos = j;
             }
         }
     }
 
-    if(srcIsASCII == FALSE){
+    if(srcIsASCII == false){
         // step 2: process the string
         b1Len = prep->process(src,srcLength,b1,b1Capacity,allowUnassigned, parseError, *status);
         if(*status == U_BUFFER_OVERFLOW_ERROR){
@@ -505,7 +518,7 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
 
         // copy everything to b1
         if(srcLength < b1Capacity){
-            uprv_memmove(b1,src, srcLength * U_SIZEOF_UCHAR);
+            u_memmove(b1, src, srcLength);
         }else{
             /* we do not have enough room so grow the buffer*/
             b1 = (UChar*) uprv_malloc(srcLength * U_SIZEOF_UCHAR);
@@ -513,7 +526,7 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
                 *status = U_MEMORY_ALLOCATION_ERROR;
                 goto CLEANUP;
             }
-            uprv_memmove(b1,src, srcLength * U_SIZEOF_UCHAR);
+            u_memmove(b1, src, srcLength);
         }
         b1Len = srcLength;
     }
@@ -575,17 +588,17 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
         //step 8: return output of step 5
         reqLength = b2Len;
         if(b2Len <= destCapacity) {
-            uprv_memmove(dest, b2, b2Len * U_SIZEOF_UCHAR);
+            u_memmove(dest, b2, b2Len);
         }
     }else{
         // verify that STD3 ASCII rules are satisfied
-        if(useSTD3ASCIIRules == TRUE){
-            if( srcIsLDH == FALSE /* source contains some non-LDH characters */
+        if(useSTD3ASCIIRules == true){
+            if( srcIsLDH == false /* source contains some non-LDH characters */
                 || src[0] ==  HYPHEN || src[srcLength-1] == HYPHEN){
                 *status = U_IDNA_STD3_ASCII_RULES_ERROR;
 
                 /* populate the parseError struct */
-                if(srcIsLDH==FALSE){
+                if(srcIsLDH==false){
                     // failPos is always set the index of failure
                     uprv_syntaxError(src,failPos, srcLength,parseError);
                 }else if(src[0] == HYPHEN){
@@ -601,7 +614,7 @@ idnaref_toUnicode(const UChar* src, int32_t srcLength,
         }
         //copy the source to destination
         if(srcLength <= destCapacity){
-            uprv_memmove(dest,src,srcLength * U_SIZEOF_UCHAR);
+            u_memmove(dest, src, srcLength);
         }
         reqLength = srcLength;
     }
@@ -626,9 +639,9 @@ CLEANUP:
         //copy the source to destination
         if(dest && srcLength <= destCapacity){
           if(srcLength == -1) {
-            uprv_memmove(dest,src,u_strlen(src)* U_SIZEOF_UCHAR);
+            u_memmove(dest, src, u_strlen(src));
           } else {
-            uprv_memmove(dest,src,srcLength * U_SIZEOF_UCHAR);
+            u_memmove(dest, src, srcLength);
           }
         }
         reqLength = srcLength;
@@ -648,7 +661,7 @@ getNextSeparator(UChar *src,int32_t srcLength,NamePrepTransform* prep,
         for(i=0 ; ;i++){
             if(src[i] == 0){
                 *limit = src + i; // point to null
-                *done = TRUE;
+                *done = true;
                 return i;
             }
             if(prep->isLabelSeparator(src[i],*status)){
@@ -668,7 +681,7 @@ getNextSeparator(UChar *src,int32_t srcLength,NamePrepTransform* prep,
         // we have not found the delimiter
         if(i==srcLength){
             *limit = src+srcLength;
-            *done = TRUE;
+            *done = true;
         }
         return i;
     }
@@ -692,7 +705,7 @@ idnaref_IDNToASCII(  const UChar* src, int32_t srcLength,
     int32_t reqLength = 0;
 //    UParseError parseError;
 
-    NamePrepTransform* prep = TestIDNA::getInstance(*status);
+    NamePrepTransform* prep = getInstance(*status);
 
     //initialize pointers to stack buffers
     UChar b1Stack[MAX_LABEL_BUFFER_SIZE];
@@ -706,7 +719,7 @@ idnaref_IDNToASCII(  const UChar* src, int32_t srcLength,
     //get the options
 //    UBool allowUnassigned   = (UBool)((options & IDNAREF_ALLOW_UNASSIGNED) != 0);
 //    UBool useSTD3ASCIIRules = (UBool)((options & IDNAREF_USE_STD3_RULES) != 0);
-    UBool done = FALSE;
+    UBool done = false;
 
     if(U_FAILURE(*status)){
         goto CLEANUP;
@@ -750,13 +763,13 @@ idnaref_IDNToASCII(  const UChar* src, int32_t srcLength,
             int32_t tempLen = (reqLength + b1Len );
             // copy to dest
             if( tempLen< destCapacity){
-                uprv_memmove(dest+reqLength, b1, b1Len * U_SIZEOF_UCHAR);
+                u_memmove(dest+reqLength, b1, b1Len);
             }
 
             reqLength = tempLen;
 
             // add the label separator
-            if(done == FALSE){
+            if(done == false){
                 if(reqLength < destCapacity){
                     dest[reqLength] = FULL_STOP;
                 }
@@ -799,13 +812,13 @@ idnaref_IDNToASCII(  const UChar* src, int32_t srcLength,
             int32_t tempLen = (reqLength + b1Len );
             // copy to dest
             if( tempLen< destCapacity){
-                uprv_memmove(dest+reqLength, b1, b1Len * U_SIZEOF_UCHAR);
+                u_memmove(dest+reqLength, b1, b1Len);
             }
 
             reqLength = tempLen;
 
             // add the label separator
-            if(done == FALSE){
+            if(done == false){
                 if(reqLength < destCapacity){
                     dest[reqLength] = FULL_STOP;
                 }
@@ -813,7 +826,7 @@ idnaref_IDNToASCII(  const UChar* src, int32_t srcLength,
             }
 
             labelStart = delimiter;
-            remainingLen = srcLength - (delimiter - src);
+            remainingLen = static_cast<int32_t>(srcLength - (delimiter - src));
         }
     }
 
@@ -846,9 +859,9 @@ idnaref_IDNToUnicode(  const UChar* src, int32_t srcLength,
 
     int32_t reqLength = 0;
 
-    UBool done = FALSE;
+    UBool done = false;
 
-    NamePrepTransform* prep = TestIDNA::getInstance(*status);
+    NamePrepTransform* prep = getInstance(*status);
 
     //initialize pointers to stack buffers
     UChar b1Stack[MAX_LABEL_BUFFER_SIZE];
@@ -876,7 +889,7 @@ idnaref_IDNToUnicode(  const UChar* src, int32_t srcLength,
 
             labelLen = getNextSeparator(labelStart, -1, prep, &delimiter, &done, status);
 
-           if(labelLen==0 && done==FALSE){
+           if(labelLen==0 && done==false){
                 *status = U_IDNA_ZERO_LENGTH_LABEL_ERROR;
             }
             b1Len = idnaref_toUnicode(labelStart, labelLen, b1, b1Capacity,
@@ -904,12 +917,12 @@ idnaref_IDNToUnicode(  const UChar* src, int32_t srcLength,
             int32_t tempLen = (reqLength + b1Len );
             // copy to dest
             if( tempLen< destCapacity){
-                uprv_memmove(dest+reqLength, b1, b1Len * U_SIZEOF_UCHAR);
+                u_memmove(dest+reqLength, b1, b1Len);
             }
 
             reqLength = tempLen;
             // add the label separator
-            if(done == FALSE){
+            if(done == false){
                 if(reqLength < destCapacity){
                     dest[reqLength] = FULL_STOP;
                 }
@@ -927,7 +940,7 @@ idnaref_IDNToUnicode(  const UChar* src, int32_t srcLength,
 
             labelLen = getNextSeparator(labelStart, remainingLen, prep, &delimiter, &done, status);
 
-            if(labelLen==0 && done==FALSE){
+            if(labelLen==0 && done==false){
                 *status = U_IDNA_ZERO_LENGTH_LABEL_ERROR;
             }
 
@@ -956,13 +969,13 @@ idnaref_IDNToUnicode(  const UChar* src, int32_t srcLength,
             int32_t tempLen = (reqLength + b1Len );
             // copy to dest
             if( tempLen< destCapacity){
-                uprv_memmove(dest+reqLength, b1, b1Len * U_SIZEOF_UCHAR);
+                u_memmove(dest+reqLength, b1, b1Len);
             }
 
             reqLength = tempLen;
 
             // add the label separator
-            if(done == FALSE){
+            if(done == false){
                 if(reqLength < destCapacity){
                     dest[reqLength] = FULL_STOP;
                 }
@@ -970,7 +983,7 @@ idnaref_IDNToUnicode(  const UChar* src, int32_t srcLength,
             }
 
             labelStart = delimiter;
-            remainingLen = srcLength - (delimiter - src);
+            remainingLen = static_cast<int32_t>(srcLength - (delimiter - src));
         }
     }
 
