@@ -1,14 +1,21 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
- * Copyright (C) 1996-2000, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "itrbnfrt.h"
 
 #include "unicode/fmtable.h"
-#include "math.h" // fabs
+#include <math.h> // fabs
+#include <stdio.h>
 
 // current macro not in icu1.8.1
 #define TESTCASE(id,test)             \
@@ -16,7 +23,7 @@
         name = #test;                 \
         if (exec) {                   \
             logln(#test "---");       \
-            logln((UnicodeString)""); \
+            logln();                  \
             test();                   \
         }                             \
         break
@@ -37,7 +44,7 @@ void RbnfRoundTripTest::runIndexedTest(int32_t index, UBool exec, const char* &n
       TESTCASE(8, TestDutchSpelloutRT);
       TESTCASE(9, TestJapaneseSpelloutRT);
       TESTCASE(10, TestRussianSpelloutRT);
-      TESTCASE(11, TestGreekSpelloutRT);
+      TESTCASE(11, TestPortugueseSpelloutRT);
 #else
       TESTCASE(0, TestRBNFDisabled);
 #endif
@@ -57,10 +64,10 @@ RbnfRoundTripTest::TestEnglishSpelloutRT()
 {
   UErrorCode status = U_ZERO_ERROR;
   RuleBasedNumberFormat* formatter
-    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::US, status);
+    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::getUS(), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, -12345678, 12345678);
   }
@@ -75,10 +82,10 @@ RbnfRoundTripTest::TestDurationsRT()
 {
   UErrorCode status = U_ZERO_ERROR;
   RuleBasedNumberFormat* formatter
-    = new RuleBasedNumberFormat(URBNF_DURATION, Locale::US, status);
+    = new RuleBasedNumberFormat(URBNF_DURATION, Locale::getUS(), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, 0, 12345678);
   }
@@ -96,7 +103,7 @@ RbnfRoundTripTest::TestSpanishSpelloutRT()
     = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("es", "es"), status);
  
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, -12345678, 12345678);
   }
@@ -114,7 +121,7 @@ RbnfRoundTripTest::TestFrenchSpelloutRT()
     = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::getFrance(), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, -12345678, 12345678);
   }
@@ -132,7 +139,7 @@ RbnfRoundTripTest::TestSwissFrenchSpelloutRT()
     = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("fr", "CH"), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, -12345678, 12345678);
   }
@@ -147,10 +154,10 @@ RbnfRoundTripTest::TestItalianSpelloutRT()
 {
   UErrorCode status = U_ZERO_ERROR;
   RuleBasedNumberFormat* formatter
-    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::ITALIAN, status);
+    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::getItalian(), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, -999999, 999999);
   }
@@ -168,7 +175,7 @@ RbnfRoundTripTest::TestGermanSpelloutRT()
     = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::getGermany(), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, 0, 12345678);
   }
@@ -186,7 +193,7 @@ RbnfRoundTripTest::TestSwedishSpelloutRT()
     = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("sv", "SE"), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, 0, 12345678);
   }
@@ -204,7 +211,7 @@ RbnfRoundTripTest::TestDutchSpelloutRT()
     = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("nl", "NL"), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, -12345678, 12345678);
   }
@@ -219,10 +226,10 @@ RbnfRoundTripTest::TestJapaneseSpelloutRT()
 {
   UErrorCode status = U_ZERO_ERROR;
   RuleBasedNumberFormat* formatter
-    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::JAPAN, status);
+    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale::getJapan(), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, 0, 12345678);
   }
@@ -240,7 +247,7 @@ RbnfRoundTripTest::TestRussianSpelloutRT()
     = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("ru", "RU"), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
     doTest(formatter, 0, 12345678);
   }
@@ -248,19 +255,19 @@ RbnfRoundTripTest::TestRussianSpelloutRT()
 }
 
 /**
- * Perform an exhaustive round-trip test on the Greek spellout rules
+ * Perform an exhaustive round-trip test on the Portuguese spellout rules
  */
 void
-RbnfRoundTripTest::TestGreekSpelloutRT() 
+RbnfRoundTripTest::TestPortugueseSpelloutRT() 
 {
   UErrorCode status = U_ZERO_ERROR;
   RuleBasedNumberFormat* formatter
-    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("el", "GR"), status);
+    = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("pt", "BR"), status);
 
   if (U_FAILURE(status)) {
-    errln("failed to construct formatter");
+    errcheckln(status, "failed to construct formatter - %s", u_errorName(status));
   } else {
-    doTest(formatter, 0, 12345678);
+    doTest(formatter, -12345678, 12345678);
   }
   delete formatter;
 }
@@ -355,3 +362,5 @@ RbnfRoundTripTest::TestRBNFDisabled() {
 
 /* U_HAVE_RBNF */
 #endif
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
