@@ -1,15 +1,18 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*********************************************************************
  * COPYRIGHT:
- * Copyright (c) 2010-2014, International Business Machines Corporation and
+ * Copyright (c) 2010-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  *********************************************************************/
 
 #include "locnmtst.h"
+#include "unicode/ustring.h"
 #include "cstring.h"
 
 /*
  Usage:
-    test_assert(    Test (should be TRUE)  )
+    test_assert(    Test (should be true)  )
 
    Example:
        test_assert(i==3);
@@ -17,17 +20,16 @@
    the macro is ugly but makes the tests pretty.
 */
 
-#define test_assert(test) \
-    { \
-        if(!(test)) \
-            errln("FAIL: " #test " was not true. In " __FILE__ " on line %d", __LINE__ ); \
-        else \
-            logln("PASS: asserted " #test); \
-    }
+#define test_assert(test) UPRV_BLOCK_MACRO_BEGIN { \
+    if(!(test)) \
+        errln("FAIL: " #test " was not true. In " __FILE__ " on line %d", __LINE__ ); \
+    else \
+        logln("PASS: asserted " #test); \
+} UPRV_BLOCK_MACRO_END
 
 /*
  Usage:
-    test_assert_print(    Test (should be TRUE),  printable  )
+    test_assert_print(    Test (should be true),  printable  )
 
    Example:
        test_assert(i==3, toString(i));
@@ -35,25 +37,25 @@
    the macro is ugly but makes the tests pretty.
 */
 
-#define test_assert_print(test,print) \
-    { \
-        if(!(test)) \
-            errln("FAIL: " #test " was not true. " + UnicodeString(print) ); \
-        else \
-            logln("PASS: asserted " #test "-> " + UnicodeString(print)); \
-    }
+#define test_assert_print(test,print) UPRV_BLOCK_MACRO_BEGIN { \
+    if(!(test)) \
+        errln("FAIL: " #test " was not true. " + UnicodeString(print) ); \
+    else \
+        logln("PASS: asserted " #test "-> " + UnicodeString(print)); \
+} UPRV_BLOCK_MACRO_END
 
-#define test_assert_equal(target,value) \
-  { \
+#define test_assert_equal(target,value) UPRV_BLOCK_MACRO_BEGIN { \
     if (UnicodeString(target)!=(value)) { \
-      logln("unexpected value '" + (value) + "'"); \
-      dataerrln("FAIL: " #target " == " #value " was not true. In " __FILE__ " on line %d", __LINE__); \
+        logln("unexpected value '" + (value) + "'"); \
+        dataerrln("FAIL: " #target " == " #value " was not true. In " __FILE__ " on line %d", __LINE__); \
     } else { \
-      logln("PASS: asserted " #target " == " #value); \
+        logln("PASS: asserted " #target " == " #value); \
     } \
-  }
+} UPRV_BLOCK_MACRO_END
 
-#define test_dumpLocale(l) { logln(#l " = " + UnicodeString(l.getName(), "")); }
+#define test_dumpLocale(l) UPRV_BLOCK_MACRO_BEGIN { \
+    logln(#l " = " + UnicodeString(l.getName(), "")); \
+} UPRV_BLOCK_MACRO_END
 
 LocaleDisplayNamesTest::LocaleDisplayNamesTest() {
 }
@@ -61,7 +63,7 @@ LocaleDisplayNamesTest::LocaleDisplayNamesTest() {
 LocaleDisplayNamesTest::~LocaleDisplayNamesTest() {
 }
 
-void LocaleDisplayNamesTest::runIndexedTest(int32_t index, UBool exec, const char* &name, 
+void LocaleDisplayNamesTest::runIndexedTest(int32_t index, UBool exec, const char* &name,
                         char* /*par*/) {
     switch (index) {
 #if !UCONFIG_NO_FORMATTING
@@ -78,6 +80,9 @@ void LocaleDisplayNamesTest::runIndexedTest(int32_t index, UBool exec, const cha
         TESTCASE(10, TestUntranslatedKeywords);
         TESTCASE(11, TestPrivateUse);
         TESTCASE(12, TestUldnDisplayContext);
+        TESTCASE(13, TestUldnWithGarbage);
+        TESTCASE(14, TestSubstituteHandling);
+        TESTCASE(15, TestNumericRegionID);
 #endif
         default:
             name = "";
@@ -188,6 +193,16 @@ void LocaleDisplayNamesTest::TestUldnOpenDialect() {
   test_assert_equal("British English", str);
 }
 
+void LocaleDisplayNamesTest::TestUldnWithGarbage() {
+  UErrorCode status = U_ZERO_ERROR;
+  const int32_t kMaxResultSize = 150;  // long enough
+  UChar result[150];
+  ULocaleDisplayNames *ldn = uldn_open(Locale::getUS().getName(), ULDN_DIALECT_NAMES, &status);
+  int32_t len = uldn_localeDisplayName(ldn, "english (United States) [w", result, kMaxResultSize, &status);
+  uldn_close(ldn);
+  test_assert(U_FAILURE(status) && len == 0);
+}
+
 void LocaleDisplayNamesTest::TestUldnWithKeywordsAndEverything() {
   UErrorCode status = U_ZERO_ERROR;
   const int32_t kMaxResultSize = 150;  // long enough
@@ -231,7 +246,7 @@ void LocaleDisplayNamesTest::TestUldnComponents() {
   }
 
   {
-    int32_t len = uldn_scriptCodeDisplayName(ldn, USCRIPT_TRADITIONAL_HAN, result, kMaxResultSize, 
+    int32_t len = uldn_scriptCodeDisplayName(ldn, USCRIPT_TRADITIONAL_HAN, result, kMaxResultSize,
                          &status);
     UnicodeString str(result, len, kMaxResultSize);
     test_assert_equal("Traditionell", str);
@@ -256,7 +271,7 @@ void LocaleDisplayNamesTest::TestUldnComponents() {
   }
 
   {
-    int32_t len = uldn_keyValueDisplayName(ldn, "calendar", "gregorian", result, 
+    int32_t len = uldn_keyValueDisplayName(ldn, "calendar", "gregorian", result,
                        kMaxResultSize, &status);
     UnicodeString str(result, len, kMaxResultSize);
     test_assert_equal("Gregorianischer Kalender", str);
@@ -275,64 +290,78 @@ typedef struct {
     const UChar * result;
 } LocNameDispContextItem;
 
-static char en[]    = "en";
+static char en[] = "en";
+static char en_cabud[] = "en@calendar=buddhist";
 static char en_GB[] = "en_GB";
+static char uz_Latn[] = "uz_Latn";
 
 static UChar daFor_en[]       = {0x65,0x6E,0x67,0x65,0x6C,0x73,0x6B,0}; //"engelsk"
+static UChar daFor_en_cabud[] = {0x65,0x6E,0x67,0x65,0x6C,0x73,0x6B,0x20,0x28,0x62,0x75,0x64,0x64,0x68,0x69,0x73,0x74,0x69,0x73,0x6B,0x20,
+                                 0x6B,0x61,0x6C,0x65,0x6E,0x64,0x65,0x72,0x29,0}; //"engelsk (buddhistisk kalender)"
 static UChar daFor_en_GB[]    = {0x65,0x6E,0x67,0x65,0x6C,0x73,0x6B,0x20,0x28,0x53,0x74,0x6F,0x72,0x62,0x72,0x69,0x74,0x61,0x6E,0x6E,0x69,0x65,0x6E,0x29,0}; //"engelsk (Storbritannien)"
-static UChar daFor_en_GB_S[]  = {0x65,0x6E,0x67,0x65,0x6C,0x73,0x6B,0x20,0x28,0x55,0x4B,0x29,0}; //"engelsk (UK)"
 static UChar daFor_en_GB_D[]  = {0x62,0x72,0x69,0x74,0x69,0x73,0x6B,0x20,0x65,0x6E,0x67,0x65,0x6C,0x73,0x6B,0}; //"britisk engelsk"
 static UChar esFor_en[]       = {0x69,0x6E,0x67,0x6C,0xE9,0x73,0}; //"ingles" with acute on the e
 static UChar esFor_en_GB[]    = {0x69,0x6E,0x67,0x6C,0xE9,0x73,0x20,0x28,0x52,0x65,0x69,0x6E,0x6F,0x20,0x55,0x6E,0x69,0x64,0x6F,0x29,0}; //"ingles (Reino Unido)" ...
 static UChar esFor_en_GB_S[]  = {0x69,0x6E,0x67,0x6C,0xE9,0x73,0x20,0x28,0x52,0x55,0x29,0}; //"ingles (RU)" ...
 static UChar esFor_en_GB_D[]  = {0x69,0x6E,0x67,0x6C,0xE9,0x73,0x20,0x62,0x72,0x69,0x74,0xE1,0x6E,0x69,0x63,0x6F,0}; //"ingles britanico" with acute on the e, a
+static UChar ruFor_uz_Latn[]  = {0x0443,0x0437,0x0431,0x0435,0x043A,0x0441,0x043A,0x0438,0x0439,0x20,0x28,0x043B,0x0430,0x0442,0x0438,0x043D,0x0438,0x0446,0x0430,0x29,0}; // all lowercase
 #if !UCONFIG_NO_BREAK_ITERATION
 static UChar daFor_en_T[]     = {0x45,0x6E,0x67,0x65,0x6C,0x73,0x6B,0}; //"Engelsk"
+static UChar daFor_en_cabudT[]= {0x45,0x6E,0x67,0x65,0x6C,0x73,0x6B,0x20,0x28,0x62,0x75,0x64,0x64,0x68,0x69,0x73,0x74,0x69,0x73,0x6B,0x20,
+                                 0x6B,0x61,0x6C,0x65,0x6E,0x64,0x65,0x72,0x29,0}; //"Engelsk (buddhistisk kalender)"
 static UChar daFor_en_GB_T[]  = {0x45,0x6E,0x67,0x65,0x6C,0x73,0x6B,0x20,0x28,0x53,0x74,0x6F,0x72,0x62,0x72,0x69,0x74,0x61,0x6E,0x6E,0x69,0x65,0x6E,0x29,0}; //"Engelsk (Storbritannien)"
-static UChar daFor_en_GB_ST[] = {0x45,0x6E,0x67,0x65,0x6C,0x73,0x6B,0x20,0x28,0x55,0x4B,0x29,0}; //"Engelsk (UK)"
 static UChar daFor_en_GB_DT[] = {0x42,0x72,0x69,0x74,0x69,0x73,0x6B,0x20,0x65,0x6E,0x67,0x65,0x6C,0x73,0x6B,0}; //"Britisk engelsk"
 static UChar esFor_en_T[]     = {0x49,0x6E,0x67,0x6C,0xE9,0x73,0}; //"Ingles" with acute on the e
 static UChar esFor_en_GB_T[]  = {0x49,0x6E,0x67,0x6C,0xE9,0x73,0x20,0x28,0x52,0x65,0x69,0x6E,0x6F,0x20,0x55,0x6E,0x69,0x64,0x6F,0x29,0}; //"Ingles (Reino Unido)" ...
 static UChar esFor_en_GB_ST[] = {0x49,0x6E,0x67,0x6C,0xE9,0x73,0x20,0x28,0x52,0x55,0x29,0}; //"Ingles (RU)" ...
 static UChar esFor_en_GB_DT[] = {0x49,0x6E,0x67,0x6C,0xE9,0x73,0x20,0x62,0x72,0x69,0x74,0xE1,0x6E,0x69,0x63,0x6F,0}; //"Ingles britanico" with acute on the e, a
+static UChar ruFor_uz_Latn_T[]= {0x0423,0x0437,0x0431,0x0435,0x043A,0x0441,0x043A,0x0438,0x0439,0x20,0x28,0x043B,0x0430,0x0442,0x0438,0x043D,0x0438,0x0446,0x0430,0x29,0}; // first char upper
 #endif /* #if !UCONFIG_NO_BREAK_ITERATION */
 
 static const LocNameDispContextItem ctxtItems[] = {
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   en,    daFor_en },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   en_cabud, daFor_en_cabud },
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB },
-    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB_S },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB },
     { "da", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB_D },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   en,    esFor_en },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_SHORT,  en_GB, esFor_en_GB_S },
     { "es", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB_D },
+    { "ru", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE,    UDISPCTX_LENGTH_FULL,   uz_Latn, ruFor_uz_Latn },
 #if !UCONFIG_NO_BREAK_ITERATION
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   en,    daFor_en_T },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   en_cabud, daFor_en_cabudT },
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB_T },
-    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB_ST },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB_T },
     { "da", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB_DT },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   en,    esFor_en_T },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB_T },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_SHORT,  en_GB, esFor_en_GB_ST },
     { "es", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB_DT },
+    { "ru", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_BEGINNING_OF_SENTENCE, UDISPCTX_LENGTH_FULL,   uz_Latn, ruFor_uz_Latn_T },
 
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   en,    daFor_en_T },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   en_cabud, daFor_en_cabudT },
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB_T },
-    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB_ST },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB_T },
     { "da", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB_DT },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   en,    esFor_en_T },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB_T },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_SHORT,  en_GB, esFor_en_GB_ST },
     { "es", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB_DT },
+    { "ru", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_UI_LIST_OR_MENU,       UDISPCTX_LENGTH_FULL,   uz_Latn, ruFor_uz_Latn_T },
 
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   en,    daFor_en },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   en_cabud, daFor_en_cabud },
     { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB },
-    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB_S },
+    { "da", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_SHORT,  en_GB, daFor_en_GB },
     { "da", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   en_GB, daFor_en_GB_D },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   en,    esFor_en_T },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB_T },
     { "es", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_SHORT,  en_GB, esFor_en_GB_ST },
     { "es", UDISPCTX_DIALECT_NAMES,  UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   en_GB, esFor_en_GB_DT },
+    { "ru", UDISPCTX_STANDARD_NAMES, UDISPCTX_CAPITALIZATION_FOR_STANDALONE,            UDISPCTX_LENGTH_FULL,   uz_Latn, ruFor_uz_Latn_T },
  #endif /* #if !UCONFIG_NO_BREAK_ITERATION */
     { NULL, (UDisplayContext)0,      (UDisplayContext)0,                                (UDisplayContext)0,     NULL,  NULL }
 };
@@ -379,7 +408,7 @@ void LocaleDisplayNamesTest::TestRootEtc() {
   UnicodeString temp;
   LocaleDisplayNames *ldn = LocaleDisplayNames::createInstance(Locale::getUS());
   const char *locname = "@collation=phonebook";
-  const char *target = "Root (Phonebook Sort Order)";
+  const char *target = "root (Phonebook Sort Order)";
   ldn->localeDisplayName(locname, temp);
   test_assert_equal(target, temp);
 
@@ -392,4 +421,178 @@ void LocaleDisplayNamesTest::TestRootEtc() {
   delete ldn;
 }
 
+void LocaleDisplayNamesTest::TestNumericRegionID() {
+    UErrorCode err = U_ZERO_ERROR;
+    ULocaleDisplayNames* ldn = uldn_open("es_MX", ULDN_STANDARD_NAMES, &err);
+    UChar displayName[200];
+    uldn_regionDisplayName(ldn, "019", displayName, 200, &err);
+    test_assert(U_SUCCESS(err));
+    test_assert_equal(UnicodeString(u"América"), UnicodeString(displayName));
+    uldn_close(ldn);    
+
+    err = U_ZERO_ERROR; // reset in case the test above returned an error code
+    ldn = uldn_open("en_AU", ULDN_STANDARD_NAMES, &err);
+    uldn_regionDisplayName(ldn, "002", displayName, 200, &err);
+    test_assert(U_SUCCESS(err));
+    test_assert_equal(UnicodeString(u"Africa"), UnicodeString(displayName));
+    uldn_close(ldn);    
+}
+
+static const char unknown_region[] = "wx";
+static const char unknown_lang[] = "xy";
+static const char unknown_script[] = "wxyz";
+static const char unknown_variant[] = "abc";
+static const char unknown_key[] = "efg";
+static const char unknown_ca_value[] = "ijk";
+static const char known_lang_unknown_script[] = "en-wxyz";
+static const char unknown_lang_unknown_script[] = "xy-wxyz";
+static const char unknown_lang_known_script[] = "xy-Latn";
+static const char unknown_lang_unknown_region[] = "xy-wx";
+static const char known_lang_unknown_region[] = "en-wx";
+static const char unknown_lang_known_region[] = "xy-US";
+static const char unknown_lang_unknown_script_unknown_region[] = "xy-wxyz-wx";
+static const char known_lang_unknown_script_unknown_region[] = "en-wxyz-wx";
+static const char unknown_lang_known_script_unknown_region[] = "xy-Latn-wx";
+static const char unknown_lang_known_script_known_region[] = "xy-wxyz-US";
+static const char known_lang[] = "en";
+static const char known_lang_known_script[] = "en-Latn";
+static const char known_lang_known_region[] = "en-US";
+static const char known_lang_known_script_known_region[] = "en-Latn-US";
+
+void LocaleDisplayNamesTest::VerifySubstitute(LocaleDisplayNames* ldn) {
+  UnicodeString temp;
+  // Ensure the default is UDISPCTX_SUBSTITUTE
+  UDisplayContext context = ldn->getContext(UDISPCTX_TYPE_SUBSTITUTE_HANDLING);
+  test_assert(UDISPCTX_SUBSTITUTE == context);
+
+  ldn->regionDisplayName(unknown_region, temp);
+  test_assert_equal(unknown_region, temp);
+  ldn->languageDisplayName(unknown_lang, temp);
+  test_assert_equal(unknown_lang, temp);
+  ldn->scriptDisplayName(unknown_script, temp);
+  test_assert_equal(unknown_script, temp);
+  ldn->variantDisplayName(unknown_variant, temp);
+  test_assert_equal(unknown_variant, temp);
+  ldn->keyDisplayName(unknown_key, temp);
+  test_assert_equal(unknown_key, temp);
+  ldn->keyValueDisplayName("ca", unknown_ca_value, temp);
+  test_assert_equal(unknown_ca_value, temp);
+
+  ldn->localeDisplayName(unknown_lang, temp);
+  test_assert_equal(unknown_lang, temp);
+  ldn->localeDisplayName(known_lang_unknown_script, temp);
+  test_assert_equal("Englisch (Wxyz)", temp);
+  ldn->localeDisplayName(unknown_lang_unknown_script, temp);
+  test_assert_equal("xy (Wxyz)", temp);
+  ldn->localeDisplayName(unknown_lang_known_script, temp);
+  test_assert_equal("xy (Lateinisch)", temp);
+  ldn->localeDisplayName(unknown_lang_unknown_region, temp);
+  test_assert_equal("xy (WX)", temp);
+  ldn->localeDisplayName(known_lang_unknown_region, temp);
+  test_assert_equal("Englisch (WX)", temp);
+  ldn->localeDisplayName(unknown_lang_known_region, temp);
+  test_assert_equal("xy (Vereinigte Staaten)", temp);
+  ldn->localeDisplayName(unknown_lang_unknown_script_unknown_region, temp);
+  test_assert_equal("xy (Wxyz, WX)", temp);
+  ldn->localeDisplayName(known_lang_unknown_script_unknown_region, temp);
+  test_assert_equal("Englisch (Wxyz, WX)", temp);
+  ldn->localeDisplayName(unknown_lang_known_script_unknown_region, temp);
+  test_assert_equal("xy (Lateinisch, WX)", temp);
+  ldn->localeDisplayName(unknown_lang_known_script_known_region, temp);
+  test_assert_equal("xy (Wxyz, Vereinigte Staaten)", temp);
+
+  ldn->localeDisplayName(known_lang, temp);
+  test_assert_equal("Englisch", temp);
+  ldn->localeDisplayName(known_lang_known_script, temp);
+  test_assert_equal("Englisch (Lateinisch)", temp);
+  ldn->localeDisplayName(known_lang_known_region, temp);
+  test_assert_equal("Englisch (Vereinigte Staaten)", temp);
+  ldn->localeDisplayName(known_lang_known_script_known_region, temp);
+  test_assert_equal("Englisch (Lateinisch, Vereinigte Staaten)", temp);
+}
+
+void LocaleDisplayNamesTest::VerifyNoSubstitute(LocaleDisplayNames* ldn) {
+  UnicodeString temp("");
+  std::string utf8;
+  // Ensure the default is UDISPCTX_SUBSTITUTE
+  UDisplayContext context = ldn->getContext(UDISPCTX_TYPE_SUBSTITUTE_HANDLING);
+  test_assert(UDISPCTX_NO_SUBSTITUTE == context);
+
+  ldn->regionDisplayName(unknown_region, temp);
+  test_assert(true == temp.isBogus());
+  ldn->languageDisplayName(unknown_lang, temp);
+  test_assert(true == temp.isBogus());
+  ldn->scriptDisplayName(unknown_script, temp);
+  test_assert(true == temp.isBogus());
+  ldn->variantDisplayName(unknown_variant, temp);
+  test_assert(true == temp.isBogus());
+  ldn->keyDisplayName(unknown_key, temp);
+  test_assert(true == temp.isBogus());
+  ldn->keyValueDisplayName("ca", unknown_ca_value, temp);
+  test_assert(true == temp.isBogus());
+
+  ldn->localeDisplayName(unknown_lang, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(known_lang_unknown_script, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(unknown_lang_unknown_script, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(unknown_lang_known_script, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(unknown_lang_unknown_region, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(known_lang_unknown_region, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(unknown_lang_known_region, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(unknown_lang_unknown_script_unknown_region, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(known_lang_unknown_script_unknown_region, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(unknown_lang_known_script_unknown_region, temp);
+  test_assert(true == temp.isBogus());
+  ldn->localeDisplayName(unknown_lang_known_script_known_region, temp);
+  test_assert(true == temp.isBogus());
+
+  ldn->localeDisplayName(known_lang, temp);
+  test_assert_equal("Englisch", temp);
+  ldn->localeDisplayName(known_lang_known_script, temp);
+  test_assert_equal("Englisch (Lateinisch)", temp);
+  ldn->localeDisplayName(known_lang_known_region, temp);
+  test_assert_equal("Englisch (Vereinigte Staaten)", temp);
+  ldn->localeDisplayName(known_lang_known_script_known_region, temp);
+  test_assert_equal("Englisch (Lateinisch, Vereinigte Staaten)", temp);
+}
+
+void LocaleDisplayNamesTest::TestSubstituteHandling() {
+  // With substitute as default
+  logln("Context: none\n");
+  std::unique_ptr<LocaleDisplayNames> ldn(LocaleDisplayNames::createInstance(Locale::getGermany()));
+  VerifySubstitute(ldn.get());
+
+  // With explicit set substitute, and standard names
+  logln("Context: UDISPCTX_SUBSTITUTE, UDISPCTX_STANDARD_NAMES\n");
+  UDisplayContext context_1[] = { UDISPCTX_SUBSTITUTE, UDISPCTX_STANDARD_NAMES };
+  ldn.reset(LocaleDisplayNames::createInstance(Locale::getGermany(), context_1, 2));
+  VerifySubstitute(ldn.get());
+
+  // With explicit set substitute and dialect names
+  logln("Context: UDISPCTX_SUBSTITUTE, UDISPCTX_DIALECT_NAMES\n");
+  UDisplayContext context_2[] = { UDISPCTX_SUBSTITUTE, UDISPCTX_DIALECT_NAMES };
+  ldn.reset(LocaleDisplayNames::createInstance(Locale::getGermany(), context_2, 2));
+  VerifySubstitute(ldn.get());
+
+  // With explicit set no_substitute, and standard names
+  logln("Context: UDISPCTX_NO_SUBSTITUTE, UDISPCTX_STANDARD_NAMES\n");
+  UDisplayContext context_3[] = { UDISPCTX_NO_SUBSTITUTE, UDISPCTX_STANDARD_NAMES };
+  ldn.reset(LocaleDisplayNames::createInstance(Locale::getGermany(), context_3, 2));
+  VerifyNoSubstitute(ldn.get());
+
+  // With explicit set no_substitute and dialect names
+  logln("Context: UDISPCTX_NO_SUBSTITUTE, UDISPCTX_DIALECT_NAMES\n");
+  UDisplayContext context_4[] = { UDISPCTX_NO_SUBSTITUTE, UDISPCTX_DIALECT_NAMES };
+  ldn.reset(LocaleDisplayNames::createInstance(Locale::getGermany(), context_4, 2));
+  VerifyNoSubstitute(ldn.get());
+}
+
 #endif   /*  UCONFIG_NO_FORMATTING */
