@@ -49,7 +49,7 @@ public:
     void TestAPI(void);
 
 protected:
-    UBool failure(UErrorCode status, const char* msg, UBool possibleDataError=FALSE);
+    UBool failure(UErrorCode status, const char* msg, UBool possibleDataError=false);
 
 };
 
