@@ -41,7 +41,7 @@
 } UPRV_BLOCK_MACRO_END
 
 #define TEST_ASSERT(expr) UPRV_BLOCK_MACRO_BEGIN { \
-    if ((expr) == FALSE) { \
+    if ((expr) == false) { \
         errln("Test Failure at file %s, line %d: \"%s\" is false.\n", __FILE__, __LINE__, #expr); \
     } \
 } UPRV_BLOCK_MACRO_END
@@ -550,8 +550,8 @@ void RBBIAPITest::TestIteration()
     // isBoundary()
     //
     bi->first();
-    if (bi->isBoundary(3) != TRUE) {
-        errln("%s:%d Incorrect value from bi->isBoundary().  Expected TRUE, got FALSE", __FILE__, __LINE__, i);
+    if (bi->isBoundary(3) != true) {
+        errln("%s:%d Incorrect value from bi->isBoundary().  Expected true, got false", __FILE__, __LINE__, i);
     }
     i = bi->current();
     if (i != 3) {
@@ -559,8 +559,8 @@ void RBBIAPITest::TestIteration()
     }
 
 
-    if (bi->isBoundary(11) != FALSE) {
-        errln("%s:%d Incorrect value from bi->isBoundary().  Expected FALSE, got TRUE", __FILE__, __LINE__, i);
+    if (bi->isBoundary(11) != false) {
+        errln("%s:%d Incorrect value from bi->isBoundary().  Expected false, got true", __FILE__, __LINE__, i);
     }
     i = bi->current();
     if (i != 10) {
@@ -727,9 +727,9 @@ void RBBIAPITest::TestRuleStatus() {
              case 2:
                  success = pos==12 && tag==UBRK_LINE_HARD; break;
              default:
-                 success = FALSE; break;
+                 success = false; break;
              }
-             if (success == FALSE) {
+             if (success == false) {
                  errln("%s:%d: incorrect line break status or position.  i=%d, pos=%d, tag=%d",
                      __FILE__, __LINE__, i, pos, tag);
                  break;
@@ -913,7 +913,7 @@ void RBBIAPITest::TestRegistration() {
 
     {
         BreakIterator* result = BreakIterator::createWordInstance("xx_XX", status);
-        UBool fail = TRUE;
+        UBool fail = true;
         if(result){
             fail = *result != *ja_word;
         }
@@ -925,7 +925,7 @@ void RBBIAPITest::TestRegistration() {
 
     {
         BreakIterator* result = BreakIterator::createCharacterInstance("ja_JP", status);
-        UBool fail = TRUE;
+        UBool fail = true;
         if(result){
             fail = *result != *ja_char;
         }
@@ -937,7 +937,7 @@ void RBBIAPITest::TestRegistration() {
 
     {
         BreakIterator* result = BreakIterator::createCharacterInstance("xx_XX", status);
-        UBool fail = TRUE;
+        UBool fail = true;
         if(result){
             fail = *result != *root_char;
         }
@@ -949,11 +949,11 @@ void RBBIAPITest::TestRegistration() {
 
     {
         StringEnumeration* avail = BreakIterator::getAvailableLocales();
-        UBool found = FALSE;
+        UBool found = false;
         const UnicodeString* p;
         while ((p = avail->snext(status))) {
             if (p->compare("xx") == 0) {
-                found = TRUE;
+                found = true;
                 break;
             }
         }
@@ -973,7 +973,7 @@ void RBBIAPITest::TestRegistration() {
     {
         BreakIterator* result = BreakIterator::createWordInstance("en_US", status);
         BreakIterator* root = BreakIterator::createWordInstance("", status);
-        UBool fail = TRUE;
+        UBool fail = true;
         if(root){
           fail = *root != *result;
         }
@@ -986,11 +986,11 @@ void RBBIAPITest::TestRegistration() {
 
     {
         StringEnumeration* avail = BreakIterator::getAvailableLocales();
-        UBool found = FALSE;
+        UBool found = false;
         const UnicodeString* p;
         while ((p = avail->snext(status))) {
             if (p->compare("xx") == 0) {
-                found = TRUE;
+                found = true;
                 break;
             }
         }
@@ -1002,15 +1002,15 @@ void RBBIAPITest::TestRegistration() {
 
     {
         int32_t count;
-        UBool   foundLocale = FALSE;
+        UBool   foundLocale = false;
         const Locale *avail = BreakIterator::getAvailableLocales(count);
         for (int i=0; i<count; i++) {
             if (avail[i] == Locale::getEnglish()) {
-                foundLocale = TRUE;
+                foundLocale = true;
                 break;
             }
         }
-        if (foundLocale == FALSE) {
+        if (foundLocale == false) {
             errln("BreakIterator::getAvailableLocales(&count), failed to find EN.");
         }
     }
@@ -1252,11 +1252,11 @@ void RBBIAPITest::TestFilteredBreakIteratorBuilder() {
 
     if (U_SUCCESS(status)) {
         logln("Adding Mr. as an exception\n");
-        TEST_ASSERT(TRUE == builder->suppressBreakAfter(ABBR_MR, status));
-        TEST_ASSERT(FALSE == builder->suppressBreakAfter(ABBR_MR, status)); // already have it
-        TEST_ASSERT(TRUE == builder->unsuppressBreakAfter(ABBR_MR, status));
-        TEST_ASSERT(FALSE == builder->unsuppressBreakAfter(ABBR_MR, status)); // already removed it
-        TEST_ASSERT(TRUE == builder->suppressBreakAfter(ABBR_MR, status));
+        TEST_ASSERT(true == builder->suppressBreakAfter(ABBR_MR, status));
+        TEST_ASSERT(false == builder->suppressBreakAfter(ABBR_MR, status)); // already have it
+        TEST_ASSERT(true == builder->unsuppressBreakAfter(ABBR_MR, status));
+        TEST_ASSERT(false == builder->unsuppressBreakAfter(ABBR_MR, status)); // already removed it
+        TEST_ASSERT(true == builder->suppressBreakAfter(ABBR_MR, status));
         TEST_ASSERT_SUCCESS(status);
 
         logln("Constructing base BI\n");
@@ -1285,8 +1285,8 @@ void RBBIAPITest::TestFilteredBreakIteratorBuilder() {
 
     if (U_SUCCESS(status)) {
         logln("Adding Mr. and Capt as an exception\n");
-        TEST_ASSERT(TRUE == builder->suppressBreakAfter(ABBR_MR, status));
-        TEST_ASSERT(TRUE == builder->suppressBreakAfter(ABBR_CAPT, status));
+        TEST_ASSERT(true == builder->suppressBreakAfter(ABBR_MR, status));
+        TEST_ASSERT(true == builder->suppressBreakAfter(ABBR_CAPT, status));
         TEST_ASSERT_SUCCESS(status);
 
         logln("Constructing base BI\n");
@@ -1318,7 +1318,7 @@ void RBBIAPITest::TestFilteredBreakIteratorBuilder() {
 
     if (U_SUCCESS(status)) {
         logln("unsuppressing 'Capt'");
-        TEST_ASSERT(TRUE == builder->unsuppressBreakAfter(ABBR_CAPT, status));
+        TEST_ASSERT(true == builder->unsuppressBreakAfter(ABBR_CAPT, status));
 
         logln("Building new BI\n");
         filteredBI.adoptInstead(builder->build(baseBI.orphan(), status));
