@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /***********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 2013-2016, International Business Machines Corporation
@@ -94,12 +96,12 @@ static KnownRegion knownRegions[] = {
     { "BS" ,  44, "029", URGN_TERRITORY, "019" },
     { "BT" ,  64, "034", URGN_TERRITORY, "142" },
     { "BU" , 104, "035", URGN_TERRITORY, "142" },
-    { "BV" ,  74, "QO" , URGN_TERRITORY, "009" },
+    { "BV" ,  74, "005", URGN_TERRITORY, "019" },
     { "BW" ,  72, "018", URGN_TERRITORY, "002" },
     { "BY" , 112, "151", URGN_TERRITORY, "150" },
     { "BZ" ,  84, "013", URGN_TERRITORY, "019" },
     { "CA" , 124, "021", URGN_TERRITORY, "019" },
-    { "CC" , 166, "QO" , URGN_TERRITORY, "009" },
+    { "CC" , 166, "053", URGN_TERRITORY, "009" },
     { "CD" , 180, "017", URGN_TERRITORY, "002" },
     { "CF" , 140, "017", URGN_TERRITORY, "002" },
     { "CG" , 178, "017", URGN_TERRITORY, "002" },
@@ -115,7 +117,7 @@ static KnownRegion knownRegions[] = {
     { "CU" , 192, "029", URGN_TERRITORY, "019" },
     { "CV" , 132, "011", URGN_TERRITORY, "002" },
     { "CW" , 531, "029", URGN_TERRITORY, "019" },
-    { "CX" , 162, "QO" , URGN_TERRITORY, "009" },
+    { "CX" , 162, "053", URGN_TERRITORY, "009" },
     { "CY" , 196, "145", URGN_TERRITORY, "142" },
     { "CZ" , 203, "151", URGN_TERRITORY, "150" },
     { "DD" , 276, "155", URGN_TERRITORY, "150" },
@@ -156,13 +158,13 @@ static KnownRegion knownRegions[] = {
     { "GP" , 312, "029", URGN_TERRITORY, "019" },
     { "GQ" , 226, "017", URGN_TERRITORY, "002" },
     { "GR" , 300, "039", URGN_TERRITORY, "150" },
-    { "GS" , 239, "QO" , URGN_TERRITORY, "009" },
+    { "GS" , 239, "005", URGN_TERRITORY, "019" },
     { "GT" , 320, "013", URGN_TERRITORY, "019" },
     { "GU" , 316, "057", URGN_TERRITORY, "009" },
     { "GW" , 624, "011", URGN_TERRITORY, "002" },
     { "GY" , 328, "005", URGN_TERRITORY, "019" },
     { "HK" , 344, "030", URGN_TERRITORY, "142" },
-    { "HM" , 334, "QO" , URGN_TERRITORY, "009" },
+    { "HM" , 334, "053", URGN_TERRITORY, "009" },
     { "HN" , 340, "013", URGN_TERRITORY, "019" },
     { "HR" , 191, "039", URGN_TERRITORY, "150" },
     { "HT" , 332, "029", URGN_TERRITORY, "019" },
@@ -173,7 +175,7 @@ static KnownRegion knownRegions[] = {
     { "IL" , 376, "145", URGN_TERRITORY, "142" },
     { "IM" , 833, "154", URGN_TERRITORY, "150" },
     { "IN" , 356, "034", URGN_TERRITORY, "142" },
-    { "IO" ,  86, "QO" , URGN_TERRITORY, "009" },
+    { "IO" ,  86, "014", URGN_TERRITORY, "002" },
     { "IQ" , 368, "145", URGN_TERRITORY, "142" },
     { "IR" , 364, "034", URGN_TERRITORY, "142" },
     { "IS" , 352, "154", URGN_TERRITORY, "150" },
@@ -288,7 +290,7 @@ static KnownRegion knownRegions[] = {
     { "TA" ,  -1, "QO",  URGN_TERRITORY, "009" },
     { "TC" , 796, "029", URGN_TERRITORY, "019" },
     { "TD" , 148, "017", URGN_TERRITORY, "002" },
-    { "TF" , 260, "QO" , URGN_TERRITORY, "009" },
+    { "TF" , 260, "014", URGN_TERRITORY, "002" },
     { "TG" , 768, "011", URGN_TERRITORY, "002" },
     { "TH" , 764, "035", URGN_TERRITORY, "142" },
     { "TJ" , 762, "143", URGN_TERRITORY, "142" },
@@ -305,7 +307,7 @@ static KnownRegion knownRegions[] = {
     { "TZ" , 834, "014", URGN_TERRITORY, "002" },
     { "UA" , 804, "151", URGN_TERRITORY, "150" },
     { "UG" , 800, "014", URGN_TERRITORY, "002" },
-    { "UM" , 581, "QO" , URGN_TERRITORY, "009" },
+    { "UM" , 581, "057", URGN_TERRITORY, "009" },
     { "US" , 840, "021", URGN_TERRITORY, "019" },
     { "UY" , 858, "005", URGN_TERRITORY, "019" },
     { "UZ" , 860, "143", URGN_TERRITORY, "142" },
@@ -356,6 +358,7 @@ RegionTest::runIndexedTest( int32_t index, UBool exec, const char* &name, char*
    TESTCASE_AUTO(TestContains);
    TESTCASE_AUTO(TestAvailableTerritories);
    TESTCASE_AUTO(TestNoContainedRegions);
+   TESTCASE_AUTO(TestGroupingChildren);
    TESTCASE_AUTO_END;
 }
 
@@ -622,11 +625,11 @@ void RegionTest::TestGetPreferredValues() {
               continue;
             }
             for ( int i = 1 ; data[i] ; i++ ) {
-                UBool found = FALSE;
+                UBool found = false;
                 preferredValues->reset(status);
                 while ( const char *check = preferredValues->next(NULL,status) ) {
                     if ( !uprv_strcmp(check,data[i]) ) {
-                        found = TRUE;
+                        found = true;
                         break;
                     }
                 }
@@ -731,6 +734,55 @@ void RegionTest::TestNoContainedRegions(void) {
   delete containedRegions;
 }
 
+void RegionTest::TestGroupingChildren(void) {
+    const char* testGroupings[] = {
+        "003", "021,013,029",
+        "419", "013,029,005",
+        "EU",  "AT,BE,CY,CZ,DE,DK,EE,ES,FI,FR,GR,HR,HU,IE,IT,LT,LU,LV,MT,NL,PL,PT,SE,SI,SK,BG,RO"
+    };
+
+    for (int32_t i = 0; i < UPRV_LENGTHOF(testGroupings); i += 2) {
+        const char* groupingCode = testGroupings[i];
+        const char* expectedChildren = testGroupings[i + 1];
+        
+        UErrorCode err = U_ZERO_ERROR;
+        const Region* grouping = Region::getInstance(groupingCode, err);
+        if (U_SUCCESS(err)) {
+            StringEnumeration* actualChildren = grouping->getContainedRegions(err);
+            if (U_SUCCESS(err)) {
+                int32_t numActualChildren = actualChildren->count(err);
+                int32_t numExpectedChildren = 0;
+                const char* expectedChildStart = expectedChildren;
+                const char* expectedChildEnd = NULL;
+                const char* actualChild = NULL;
+                while ((actualChild = actualChildren->next(NULL, err)) != NULL && *expectedChildStart != '\0') {
+                    expectedChildEnd = uprv_strchr(expectedChildStart, ',');
+                    if (expectedChildEnd == NULL) {
+                        expectedChildEnd = expectedChildStart + uprv_strlen(expectedChildStart);
+                    }
+                    if (uprv_strlen(actualChild) != size_t(expectedChildEnd - expectedChildStart) || uprv_strncmp(actualChild, expectedChildStart, expectedChildEnd - expectedChildStart) != 0) {
+                        errln("Mismatch in child list for %s at position %d: expected %s, got %s\n", groupingCode, numExpectedChildren, expectedChildStart, actualChild);
+                    }
+                    expectedChildStart = (*expectedChildEnd != '\0') ? expectedChildEnd + 1 : expectedChildEnd;
+                    ++numExpectedChildren;
+                }
+                while (expectedChildEnd != NULL && *expectedChildEnd != '\0') {
+                    expectedChildEnd = uprv_strchr(expectedChildEnd + 1, ',');
+                    ++numExpectedChildren;
+                }
+                if (numExpectedChildren != numActualChildren) {
+                    errln("Wrong number of children for %s: expected %d, got %d\n", groupingCode, numExpectedChildren, numActualChildren);
+                }
+                delete actualChildren;
+            } else {
+                errln("Couldn't create iterator for children of %s\n", groupingCode);
+            }
+        } else {
+            errln("Region %s not found\n", groupingCode);
+        }
+    }
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
 //eof
