@@ -19,7 +19,7 @@
  * Test basic functionality of various API functions
  **/
 class IntlTestSimpleDateFormatAPI : public IntlTest {
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );  
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ) override;
 
 private:
     /**
