@@ -21,6 +21,7 @@
 
 #include "tmsgfmt.h"
 #include "cmemory.h"
+#include "loctest.h"
 
 #include "unicode/format.h"
 #include "unicode/decimfmt.h"
@@ -71,6 +72,9 @@ TestMessageFormat::runIndexedTest(int32_t index, UBool exec,
     TESTCASE_AUTO(TestSelectOrdinal);
     TESTCASE_AUTO(TestDecimals);
     TESTCASE_AUTO(TestArgIsPrefixOfAnother);
+    TESTCASE_AUTO(TestMessageFormatNumberSkeleton);
+    TESTCASE_AUTO(TestMessageFormatDateSkeleton);
+    TESTCASE_AUTO(TestMessageFormatTimeSkeleton);
     TESTCASE_AUTO_END;
 }
 
@@ -181,7 +185,7 @@ void TestMessageFormat::testBug2()
     // {sfb} use double format in pattern, so result will match (not strictly necessary)
     const UnicodeString pattern = "There {0,choice,0#are no files|1#is one file|1<are {0, number} files} on disk {1}. ";
     logln("The input pattern : " + pattern);
-    MessageFormat *fmt = new MessageFormat(pattern, status);
+    LocalPointer<MessageFormat> fmt(new MessageFormat(pattern, status));
     if (U_FAILURE(status)) {
         dataerrln("MessageFormat pattern creation failed. - %s", u_errorName(status));
         return;
@@ -190,7 +194,6 @@ void TestMessageFormat::testBug2()
     if (pattern != result) {
         errln("MessageFormat::toPattern() failed.");
     }
-    delete fmt;
 }
 
 #if 0
@@ -281,25 +284,25 @@ void TestMessageFormat::PatternTest()
     };*/
 
     UnicodeString testResultStrings[] = {
-        "Quotes ', {, 'a' 1 {0}",
-        "Quotes ', {, 'a' 1 {0}",
-        "{1,number,'#',##} #34,56",
-        "There are 3,456 files on Disk at 1/12/70, 5:46 AM.",
-        "On Disk, there are 3,456 files, with $1.00.",
-        "{1,number,percent}, 345,600%,",
-        "{1,date,full}, Wednesday, December 31, 1969,",
-        "{3,date,full}, Monday, January 12, 1970,",
-        "{1,number,#,##} 34,56"
+        u"Quotes ', {, 'a' 1 {0}",
+        u"Quotes ', {, 'a' 1 {0}",
+        u"{1,number,'#',##} #34,56",
+        u"There are 3,456 files on Disk at 1/12/70, 5:46\u202FAM.",
+        u"On Disk, there are 3,456 files, with $1.00.",
+        u"{1,number,percent}, 345,600%,",
+        u"{1,date,full}, Wednesday, December 31, 1969,",
+        u"{3,date,full}, Monday, January 12, 1970,",
+        u"{1,number,#,##} 34,56"
     };
 
 
     for (int32_t i = 0; i < 9; ++i) {
         //it_out << "\nPat in:  " << testCases[i]);
 
-        MessageFormat *form = 0;
+        LocalPointer<MessageFormat> form;
         UErrorCode success = U_ZERO_ERROR;
         UnicodeString buffer;
-        form = new MessageFormat(testCases[i], Locale::getUS(), success);
+        form.adoptInstead(new MessageFormat(testCases[i], Locale::getUS(), success));
         if (U_FAILURE(success)) {
             dataerrln("MessageFormat creation failed.#1 - %s", u_errorName(success));
             logln(((UnicodeString)"MessageFormat for ") + testCases[i] + " creation failed.\n");
@@ -330,10 +333,12 @@ void TestMessageFormat::PatternTest()
             continue;
         }
         if (result != testResultStrings[i]) {
-            errln("TestMessageFormat::PatternTest failed test #4");
-            logln("TestMessageFormat::PatternTest failed #4.");
-            logln(UnicodeString("    Result: ") + result );
-            logln(UnicodeString("  Expected: ") + testResultStrings[i] );
+            errln(UnicodeString("TestMessageFormat::PatternTest failed test #4.") +
+                UnicodeString("\n    Result: ") + result +
+                UnicodeString("\n  Expected: ") + testResultStrings[i]);
+            logln(UnicodeString("TestMessageFormat::PatternTest failed test #4.") +
+                UnicodeString("\n    Result: ") + result +
+                UnicodeString("\n  Expected: ") + testResultStrings[i]);
         }
 
 
@@ -353,18 +358,17 @@ void TestMessageFormat::PatternTest()
         } else if (parseCount != count) {
             errln("MSG count not %d as expected. Got %d", count, parseCount);
         }
-        UBool failed = FALSE;
+        UBool failed = false;
         for (int32_t j = 0; j < parseCount; ++j) {
              if (values == 0 || testArgs[j] != values[j]) {
                 errln(((UnicodeString)"MSG testargs[") + j + "]: " + toString(testArgs[j]));
                 errln(((UnicodeString)"MSG values[") + j + "]  : " + toString(values[j]));
-                failed = TRUE;
+                failed = true;
              }
         }
         if (failed)
             errln("MessageFormat failed test #6");
 #endif
-        delete form;
     }
 }
 
@@ -414,11 +418,11 @@ void TestMessageFormat::testStaticFormat()
     }
 
     const UnicodeString expected(
-            "At 12:20:00 PM on Aug 8, 1997, there was a disturbance in the Force on planet 7.", "");
+            u"At 12:20:00\u202FPM on Aug 8, 1997, there was a disturbance in the Force on planet 7.");
     if (result != expected) {
-        errln("TestMessageFormat::testStaticFormat failed on test");
-        logln( UnicodeString("     Result: ") + result );
-        logln( UnicodeString("   Expected: ") + expected );
+        errln(UnicodeString("TestMessageFormat::testStaticFormat failed on test") +
+            UnicodeString("\n     Result: ") + result +
+            UnicodeString("\n   Expected: ") + expected );
     }
 }
 
@@ -672,13 +676,13 @@ MessageFormat* TestMessageFormat::internalCreate(
         UnicodeString pattern ,Locale locale ,UErrorCode &status ,  char* errMsg)
 {
     //Create the MessageFormat with simple SelectFormat
-    MessageFormat* msgFmt = new MessageFormat(pattern, locale, status);
+    LocalPointer<MessageFormat> msgFmt(new MessageFormat(pattern, locale, status));
     if (U_FAILURE(status)) {
         dataerrln( "%s error while constructing with ErrorCode as %s" ,errMsg, u_errorName(status) );
         logln(UnicodeString("TestMessageFormat::testMsgFormatSelect #1 with error code ")+(int32_t)status);
-        return NULL;
+        return nullptr;
     }
-    return msgFmt;
+    return msgFmt.orphan();
 }
 
 void TestMessageFormat::testMsgFormatSelect(/* char* par */)
@@ -805,7 +809,8 @@ void TestMessageFormat::testMsgFormatSelect(/* char* par */)
     //Select, plural, and number formats heavily nested 
     UnicodeString t6("{0} und {1, select, female {{2, plural, one {{3, select, female {ihre Freundin} other {ihr Freund}} } other {ihre {2, number, integer} {3, select, female {Freundinnen} other {Freunde}} } }} other{{2, plural, one {{3, select, female {seine Freundin} other {sein Freund}}} other {seine {2, number, integer} {3, select, female {Freundinnen} other {Freunde}}}}} } gingen nach Paris.");
     //Create the MessageFormat with Select, plural, and number formats heavily nested  
-    MessageFormat* msgFmt6 = internalCreate(t6, Locale("de"),err,(char*)"From TestMessageFormat::TestSelectFormat create t6");
+    LocalPointer<MessageFormat> msgFmt6(
+            internalCreate(t6, Locale("de"),err,(char*)"From TestMessageFormat::TestSelectFormat create t6"));
     if (!U_FAILURE(err)) {
         //Arguments 
         Formattable testArgs10[] = {"Kirti","other",(int32_t)1,"other"}; 
@@ -844,10 +849,9 @@ void TestMessageFormat::testMsgFormatSelect(/* char* par */)
         };
         //Format
         for( int i=0; i< 14; i++){
-            internalFormat( msgFmt6 , testArgs[i], 4, exp[i] ,(char*)"From TestMessageFormat::testSelectFormat format t6");
+            internalFormat( msgFmt6.getAlias(), testArgs[i], 4, exp[i] ,(char*)"From TestMessageFormat::testSelectFormat format t6");
         }
     }
-    delete msgFmt6;
 }
 
 //---------------------------------
@@ -919,7 +923,7 @@ void TestMessageFormat::testClone()
     MessageFormat *x = new MessageFormat("There are {0} files on {1}", success);
     MessageFormat *z = new MessageFormat("There are {0} files on {1} created", success);
     MessageFormat *y = 0;
-    y = (MessageFormat*)x->clone();
+    y = x->clone();
     if ( (*x == *y) && 
          (*x != *z) && 
          (*y != *z) )
@@ -999,8 +1003,7 @@ void TestMessageFormat::testSetLocale()
     UnicodeString compareStrGer = "At <time> on 08.08.1997, you made a deposit of ";
     compareStrGer += "456,83";
     compareStrGer += (UChar) 0x00a0;
-    compareStrGer += (UChar) 0x00a4;
-    compareStrGer += ".";
+    compareStrGer += "XXX.";
 
     MessageFormat msg( formatStr, err);
     result = "";
@@ -1014,21 +1017,23 @@ void TestMessageFormat::testSetLocale()
 
     logln(result);
     if (result != compareStrEng) {
-        dataerrln("***  MSG format err. - %s", u_errorName(err));
+        char bbuf[96];
+        result.extract(0, result.length(), bbuf, sizeof(bbuf));
+        dataerrln("***  MSG format err. - %s; result was %s", u_errorName(err), bbuf);
     }
 
     msg.setLocale(Locale::getEnglish());
-    UBool getLocale_ok = TRUE;
+    UBool getLocale_ok = true;
     if (msg.getLocale() != Locale::getEnglish()) {
-        errln("*** MSG getLocal err.");
-        getLocale_ok = FALSE;
+        errln("*** MSG getLocale err.");
+        getLocale_ok = false;
     }
 
     msg.setLocale(Locale::getGerman());
 
     if (msg.getLocale() != Locale::getGerman()) {
         errln("*** MSG getLocal err.");
-        getLocale_ok = FALSE;
+        getLocale_ok = false;
     }
 
     msg.applyPattern( formatStr, err);
@@ -1083,7 +1088,7 @@ void TestMessageFormat::testFormat()
     result = msg.format(
         *fmt,
         result,
-        //FieldPosition(0),
+        //FieldPosition(FieldPosition::DONT_CARE),
         fp,
         err);
 
@@ -1097,7 +1102,7 @@ void TestMessageFormat::testFormat()
     result = msg.format(
         ft_arr,
         result,
-        //FieldPosition(0),
+        //FieldPosition(FieldPosition::DONT_CARE),
         fp,
         err);
 
@@ -1244,14 +1249,14 @@ void TestMessageFormat::testAdopt()
     }
 
     UBool diff;
-    diff = TRUE;
+    diff = true;
     for (i = 0; i < count; i++) {
         a = formatsChg[i];
         b = formatsCmp[i];
         if ((a != NULL) && (b != NULL)) {
             if (*a == *b) {
                 logln("formatsChg == formatsCmp at index %d", i);
-                diff = FALSE;
+                diff = false;
             }
         }
     }
@@ -1430,8 +1435,8 @@ static void _testCopyConstructor2()
         goto cleanup;
     }
 
-    fmt3 = (MessageFormat*) fmt1->clone();
-    fmt4 = (MessageFormat*) fmt2->clone();
+    fmt3 = fmt1->clone();
+    fmt4 = fmt2->clone();
 
     if (fmt3 == NULL) {
         it_err("testCopyConstructor2: (fmt3 != NULL)");
@@ -1459,7 +1464,7 @@ void TestMessageFormat::testCopyConstructor2() {
 }
 
 /**
- * Verify that MessageFormat accomodates more than 10 arguments and
+ * Verify that MessageFormat accommodates more than 10 arguments and
  * more than 10 subformats.
  */
 void TestMessageFormat::TestUnlimitedArgsAndSubformats() {
@@ -1495,13 +1500,13 @@ void TestMessageFormat::TestUnlimitedArgsAndSubformats() {
     Formattable ARGS_OBJ(ARGS, ARGS_LENGTH);
 
     UnicodeString expected =
-        "On Nov 20, 2286 (aka 11/20/86, aka November 20, 2286) "
-        "at 9:46:40 AM (aka 9:46 AM, aka 9:46:40 AM PST) "
-        "there were 1,303 werjes "
-        "(a 8% increase over 1,202) "
-        "despite the Glimmung's efforts "
-        "and to delight of the printers, Nick, his father, "
-        "his mother, the spiddles, and of course Horace.";
+        u"On Nov 20, 2286 (aka 11/20/86, aka November 20, 2286) "
+        u"at 9:46:40\u202FAM (aka 9:46\u202FAM, aka 9:46:40\u202FAM PST) "
+        u"there were 1,303 werjes "
+        u"(a 8% increase over 1,202) "
+        u"despite the Glimmung's efforts "
+        u"and to delight of the printers, Nick, his father, "
+        u"his mother, the spiddles, and of course Horace.";
     UnicodeString result;
     msg.format(ARGS_OBJ, result, ec);
     if (result == expected) {
@@ -1800,11 +1805,11 @@ void TestMessageFormat::testCoverage(void) {
 void TestMessageFormat::testGetFormatNames() {
     IcuTestErrorCode errorCode(*this, "testGetFormatNames");
     MessageFormat msgfmt("Hello, {alice,number} {oops,date,full}  {zip,spellout} World.", Locale::getRoot(), errorCode);
-    if(errorCode.logDataIfFailureAndReset("MessageFormat() failed")) {
+    if(errorCode.errDataIfFailureAndReset("MessageFormat() failed")) {
         return;
     }
     LocalPointer<StringEnumeration> names(msgfmt.getFormatNames(errorCode));
-    if(errorCode.logIfFailureAndReset("msgfmt.getFormatNames() failed")) {
+    if(errorCode.errIfFailureAndReset("msgfmt.getFormatNames() failed")) {
         return;
     }
     const UnicodeString *name;
@@ -1846,7 +1851,7 @@ void TestMessageFormat::TestTrimArgumentName() {
     // ICU 4.8 allows and ignores white space around argument names and numbers.
     IcuTestErrorCode errorCode(*this, "TestTrimArgumentName");
     MessageFormat m("a { 0 , number , '#,#'#.0 } z", Locale::getEnglish(), errorCode);
-    if (errorCode.logDataIfFailureAndReset("Unable to instantiate MessageFormat")) {
+    if (errorCode.errDataIfFailureAndReset("Unable to instantiate MessageFormat")) {
         return;
     }
     Formattable args[1] = { (int32_t)2 };
@@ -1871,28 +1876,28 @@ void TestMessageFormat::TestSelectOrdinal() {
         "{0,plural,one{1 file}other{# files}}, "
         "{0,selectordinal,one{#st file}two{#nd file}few{#rd file}other{#th file}}",
         Locale::getEnglish(), errorCode);
-    if (errorCode.logDataIfFailureAndReset("Unable to instantiate MessageFormat")) {
+    if (errorCode.errDataIfFailureAndReset("Unable to instantiate MessageFormat")) {
         return;
     }
     Formattable args[1] = { (int32_t)21 };
     FieldPosition ignore(FieldPosition::DONT_CARE);
     UnicodeString result;
     assertEquals("plural-and-ordinal format(21) failed", "21 files, 21st file",
-                 m.format(args, 1, result, ignore, errorCode), TRUE);
+                 m.format(args, 1, result, ignore, errorCode), true);
 
     args[0].setLong(2);
     assertEquals("plural-and-ordinal format(2) failed", "2 files, 2nd file",
-                 m.format(args, 1, result.remove(), ignore, errorCode), TRUE);
+                 m.format(args, 1, result.remove(), ignore, errorCode), true);
 
     args[0].setLong(1);
     assertEquals("plural-and-ordinal format(1) failed", "1 file, 1st file",
-                 m.format(args, 1, result.remove(), ignore, errorCode), TRUE);
+                 m.format(args, 1, result.remove(), ignore, errorCode), true);
 
     args[0].setLong(3);
     assertEquals("plural-and-ordinal format(3) failed", "3 files, 3rd file",
-                 m.format(args, 1, result.remove(), ignore, errorCode), TRUE);
+                 m.format(args, 1, result.remove(), ignore, errorCode), true);
 
-    errorCode.logDataIfFailureAndReset("");
+    errorCode.errDataIfFailureAndReset("");
 }
 
 void TestMessageFormat::TestDecimals() {
@@ -1905,12 +1910,12 @@ void TestMessageFormat::TestDecimals() {
     FieldPosition ignore;
     UnicodeString result;
     assertEquals("simple format(1)", "one meter",
-            m.format(args, 1, result, ignore, errorCode), TRUE);
+            m.format(args, 1, result, ignore, errorCode), true);
 
     args[0] = (double)1.5;
     result.remove();
     assertEquals("simple format(1.5)", "1.5 meters",
-            m.format(args, 1, result, ignore, errorCode), TRUE);
+            m.format(args, 1, result, ignore, errorCode), true);
 
     // Simple but explicit.
     MessageFormat m0(
@@ -1919,12 +1924,12 @@ void TestMessageFormat::TestDecimals() {
     args[0] = (int32_t)1;
     result.remove();
     assertEquals("explicit format(1)", "one meter",
-            m0.format(args, 1, result, ignore, errorCode), TRUE);
+            m0.format(args, 1, result, ignore, errorCode), true);
 
     args[0] = (double)1.5;
     result.remove();
     assertEquals("explicit format(1.5)", "1.5 meters",
-            m0.format(args, 1, result, ignore, errorCode), TRUE);
+            m0.format(args, 1, result, ignore, errorCode), true);
 
     // With offset and specific simple format with optional decimals.
     MessageFormat m1(
@@ -1933,17 +1938,17 @@ void TestMessageFormat::TestDecimals() {
     args[0] = (int32_t)1;
     result.remove();
     assertEquals("offset format(1)", "01 meters",
-            m1.format(args, 1, result, ignore, errorCode), TRUE);
+            m1.format(args, 1, result, ignore, errorCode), true);
 
     args[0] = (int32_t)2;
     result.remove();
     assertEquals("offset format(1)", "another meter",
-            m1.format(args, 1, result, ignore, errorCode), TRUE);
+            m1.format(args, 1, result, ignore, errorCode), true);
 
     args[0] = (double)2.5;
     result.remove();
     assertEquals("offset format(1)", "02.5 meters",
-            m1.format(args, 1, result, ignore, errorCode), TRUE);
+            m1.format(args, 1, result, ignore, errorCode), true);
 
     // With offset and specific simple format with forced decimals.
     MessageFormat m2(
@@ -1952,17 +1957,17 @@ void TestMessageFormat::TestDecimals() {
     args[0] = (int32_t)1;
     result.remove();
     assertEquals("offset-decimals format(1)", "1.0 meters",
-            m2.format(args, 1, result, ignore, errorCode), TRUE);
+            m2.format(args, 1, result, ignore, errorCode), true);
 
     args[0] = (int32_t)2;
     result.remove();
     assertEquals("offset-decimals format(1)", "2.0 meters",
-            m2.format(args, 1, result, ignore, errorCode), TRUE);
+            m2.format(args, 1, result, ignore, errorCode), true);
 
     args[0] = (double)2.5;
     result.remove();
     assertEquals("offset-decimals format(1)", "2.5 meters",
-            m2.format(args, 1, result, ignore, errorCode), TRUE);
+            m2.format(args, 1, result, ignore, errorCode), true);
     errorCode.reset();
 }
 
@@ -1993,4 +1998,81 @@ void TestMessageFormat::TestArgIsPrefixOfAnother() {
     assertEquals("aa aaa", "AB ABC", mf3.format(argNames + 1, args + 1, 2, result.remove(), errorCode));
 }
 
+void TestMessageFormat::TestMessageFormatNumberSkeleton() {
+    IcuTestErrorCode status(*this, "TestMessageFormatNumberSkeleton");
+
+    static const struct TestCase {
+        const char16_t* messagePattern;
+        const char* localeName;
+        double arg;
+        const char16_t* expected;
+    } cases[] = {
+            { u"{0,number,::percent}", "en", 50, u"50%" },
+            { u"{0,number,::percent scale/100}", "en", 0.5, u"50%" },
+            { u"{0,number,   ::   percent   scale/100   }", "en", 0.5, u"50%" },
+            { u"{0,number,::currency/USD}", "en", 23, u"$23.00" },
+            { u"{0,number,::precision-integer}", "en", 514.23, u"514" },
+            { u"{0,number,::.000}", "en", 514.23, u"514.230" },
+            { u"{0,number,::.}", "en", 514.23, u"514" },
+            { u"{0,number,::}", "fr", 514.23, u"514,23" },
+            { u"Cost: {0,number,::currency/EUR}.", "en", 4.3, u"Cost: €4.30." },
+            { u"{0,number,'::'0.00}", "en", 50, u"::50.00" }, // pattern literal
+    };
+
+    for (auto& cas : cases) {
+        status.setScope(cas.messagePattern);
+        MessageFormat msgf(cas.messagePattern, cas.localeName, status);
+        UnicodeString sb;
+        FieldPosition fpos(FieldPosition::DONT_CARE);
+        Formattable argsArray[] = {{cas.arg}};
+        Formattable args(argsArray, 1);
+        msgf.format(args, sb, status);
+
+        assertEquals(cas.messagePattern, cas.expected, sb);
+    }
+}
+
+void TestMessageFormat::doTheRealDateTimeSkeletonTesting(UDate testDate,
+        const char16_t* messagePattern, const char* localeName, const char16_t* expected,
+        IcuTestErrorCode& status) {
+
+    status.setScope(messagePattern);
+    MessageFormat msgf(messagePattern, localeName, status);
+    UnicodeString sb;
+    FieldPosition fpos(FieldPosition::DONT_CARE);
+    Formattable argsArray[] = { Formattable(testDate, Formattable::kIsDate) };
+    Formattable args(argsArray, 1);
+    msgf.format(args, sb, status);
+
+    assertEquals(messagePattern, expected, sb);
+}
+
+void TestMessageFormat::TestMessageFormatDateSkeleton() {
+    IcuTestErrorCode status(*this, "TestMessageFormatDateSkeleton");
+
+    UDate date = LocaleTest::date(2021-1900, UCAL_NOVEMBER, 23, 16, 42, 55);
+
+    doTheRealDateTimeSkeletonTesting(date, u"{0,date,::MMMMd}", "en", u"November 23", status);
+    doTheRealDateTimeSkeletonTesting(date, u"{0,date,::yMMMMdjm}", "en", u"November 23, 2021 at 4:42\u202FPM", status);
+    doTheRealDateTimeSkeletonTesting(date, u"{0,date,   ::   yMMMMd   }", "en", u"November 23, 2021", status);
+    doTheRealDateTimeSkeletonTesting(date, u"{0,date,::yMMMMd}", "fr", u"23 novembre 2021", status);
+    doTheRealDateTimeSkeletonTesting(date, u"Expiration: {0,date,::yMMM}!", "en", u"Expiration: Nov 2021!", status);
+    // pattern literal
+    doTheRealDateTimeSkeletonTesting(date, u"{0,date,'::'yMMMMd}", "en", u"::2021November23", status);
+}
+
+void TestMessageFormat::TestMessageFormatTimeSkeleton() {
+    IcuTestErrorCode status(*this, "TestMessageFormatTimeSkeleton");
+
+    UDate date = LocaleTest::date(2021-1900, UCAL_NOVEMBER, 23, 16, 42, 55);
+
+    doTheRealDateTimeSkeletonTesting(date, u"{0,time,::MMMMd}", "en", u"November 23", status);
+    doTheRealDateTimeSkeletonTesting(date, u"{0,time,::yMMMMdjm}", "en", u"November 23, 2021 at 4:42\u202FPM", status);
+    doTheRealDateTimeSkeletonTesting(date, u"{0,time,   ::   yMMMMd   }", "en", u"November 23, 2021", status);
+    doTheRealDateTimeSkeletonTesting(date, u"{0,time,::yMMMMd}", "fr", u"23 novembre 2021", status);
+    doTheRealDateTimeSkeletonTesting(date, u"Expiration: {0,time,::yMMM}!", "en", u"Expiration: Nov 2021!", status);
+    // pattern literal
+    doTheRealDateTimeSkeletonTesting(date, u"{0,time,'::'yMMMMd}", "en", u"::2021November23", status);
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
