@@ -20,7 +20,7 @@
  * Tests for DecimalFormatSymbols
  **/
 class IntlTestDecimalFormatSymbols: public IntlTest {
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );  
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ) override;
 
 private:
     /**
