@@ -17,7 +17,7 @@
  **/
 class PUtilTest : public IntlTest {
     // IntlTest override
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par ) override;
 public:
 
 //    void testIEEEremainder(void);
