@@ -18,7 +18,7 @@
 
 class TimeZoneOffsetLocalTest : public IntlTest {
     // IntlTest override
-    void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par);
+    void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par) override;
 
     void TestGetOffsetAroundTransition(void);
 };
