@@ -1,9 +1,13 @@
 /*
-**********************************************************************
-* Copyright (c) 2002-2005, International Business Machines
+***********************************************************************
+* © 2016 and later: Unicode, Inc. and others.
+* License & terms of use: http://www.unicode.org/copyright.html
+***********************************************************************
+***********************************************************************
+* Copyright (c) 2002-2014, International Business Machines
 * Corporation and others.  All Rights Reserved.
-**********************************************************************
-**********************************************************************
+***********************************************************************
+***********************************************************************
 */
 #ifndef _CONVPERF_H
 #define _CONVPERF_H
@@ -14,12 +18,12 @@
 #include "unicode/ucnv.h"
 #include "unicode/uclean.h"
 #include "unicode/ustring.h"
+#include "cmemory.h" // for UPRV_LENGTHOF
 
 #include "unicode/uperf.h"
 
 #define CONVERSION_FLAGS (0) /*WC_DEFAULTCHAR WC_COMPOSITECHECK & WC_SEPCHARS*/
 #define MAX_BUF_SIZE  3048
-#define LENGTHOF(array) (sizeof(array)/sizeof((array)[0]))
 
 class ICUToUnicodePerfFunction : public UPerfFunction{
 private:
@@ -56,7 +60,7 @@ public:
         const char* mySrc = src;
         const char* sourceLimit = src + srcLen;
         UChar* myTarget = target;
-        ucnv_toUnicode(conv, &myTarget, targetLimit, &mySrc, sourceLimit, NULL, TRUE, status);
+        ucnv_toUnicode(conv, &myTarget, targetLimit, &mySrc, sourceLimit, NULL, true, status);
     }
     virtual long getOperationsPerIteration(void){
         return srcLen;
@@ -102,7 +106,7 @@ public:
         const UChar* mySrc = src;
         const UChar* sourceLimit = src + srcLen;
         char* myTarget = target;
-        ucnv_fromUnicode(conv,&myTarget, targetLimit, &mySrc, sourceLimit, NULL, TRUE, status);
+        ucnv_fromUnicode(conv,&myTarget, targetLimit, &mySrc, sourceLimit, NULL, true, status);
     }
     virtual long getOperationsPerIteration(void){
         return srcLen;
@@ -159,7 +163,7 @@ public:
         name = cpName;
         src = pszIn;
         srcLen = szLen;
-        dstLen = LENGTHOF(dest);
+        dstLen = UPRV_LENGTHOF(dest);
         unsigned short bEnc[30]={'\0'};
         const char* tenc=name;
         for(int i=0;*tenc!='\0';i++){
@@ -213,8 +217,8 @@ public:
         name = cpName;
         src = pszIn;
         srcLen = szLen;
-        dstLen = LENGTHOF(dest);
-        lpUsedDefaultChar=FALSE;
+        dstLen = UPRV_LENGTHOF(dest);
+        lpUsedDefaultChar=false;
         unsigned short bEnc[30]={'\0'};
         const char* tenc=name;
         for(int i=0;*tenc!='\0';i++){
@@ -308,7 +312,7 @@ public:
         getErr(err,status);
         src = source;
         srcLen = sourceLen;
-        dstLen = LENGTHOF(dst);
+        dstLen = UPRV_LENGTHOF(dst);
         cpName = name;
     }
 
@@ -364,7 +368,7 @@ public:
         getErr(err,status);
         src = source;
         srcLen = sourceLen;
-        dstLen = LENGTHOF(dst);
+        dstLen = UPRV_LENGTHOF(dst);
         cpName = name;
 
     }
@@ -404,7 +408,7 @@ public:
 
         src = source;
         srcLen = sourceLen;
-        dstLen = LENGTHOF(dst);
+        dstLen = UPRV_LENGTHOF(dst);
         cpName = name;
         unsigned short bEnc[30]={'\0'};
         const char* tenc=name;
@@ -465,7 +469,7 @@ public:
         }
         src = source;
         srcLen = sourceLen;
-        dstLen = LENGTHOF(dst);
+        dstLen = UPRV_LENGTHOF(dst);
         cpName = name;
         /* get the charset info */
         MIMECSETINFO mimeInfo;
