@@ -1,22 +1,25 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 ********************************************************************************
 *
-*   Copyright (C) 1996-2011, International Business Machines
+*   Copyright (C) 1996-2014, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ********************************************************************************
 */
+#include <assert.h>
+#include <ctype.h>
+#include <stdarg.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
-#include <stdarg.h>
-#include <ctype.h>
 
 #include "unicode/utrace.h"
 #include "unicode/uclean.h"
-#include "umutex.h"
 #include "putilimp.h"
+#include "udbgutil.h"
 
 /* NOTES:
    3/20/1999 srl - strncpy called w/o setting nulls at the end
@@ -89,6 +92,7 @@ static void help ( const char *argv0 );
  */
 static void vlog_err(const char *prefix, const char *pattern, va_list ap);
 static void vlog_verbose(const char *prefix, const char *pattern, va_list ap);
+static UBool vlog_knownIssue(const char *ticket, const char *pattern, va_list ap);
 
 /**
  * Log test structure, with indent
@@ -110,8 +114,11 @@ static int ERROR_COUNT = 0; /* Count of errors from all tests. */
 static int ONE_ERROR = 0; /* were there any other errors? */
 static int DATA_ERROR_COUNT = 0; /* count of data related errors or warnings */
 static int INDENT_LEVEL = 0;
-static UBool ON_LINE = FALSE; /* are we on the top line with our test name? */
-static UBool HANGING_OUTPUT = FALSE; /* did the user leave us without a trailing \n ? */
+static UBool NO_KNOWN = false;
+static void *knownList = NULL;
+static char gTestName[1024] = "";
+static UBool ON_LINE = false; /* are we on the top line with our test name? */
+static UBool HANGING_OUTPUT = false; /* did the user leave us without a trailing \n ? */
 static int GLOBAL_PRINT_COUNT = 0; /* global count of printouts */
 int REPEAT_TESTS_INIT = 0; /* Was REPEAT_TESTS initialized? */
 int REPEAT_TESTS = 1; /* Number of times to run the test */
@@ -120,13 +127,13 @@ int ERR_MSG =1; /* error messages will be displayed by default*/
 int QUICK = 1;  /* Skip some of the slower tests? */
 int WARN_ON_MISSING_DATA = 0; /* Reduce data errs to warnings? */
 UTraceLevel ICU_TRACE = UTRACE_OFF;  /* ICU tracing level */
+int WRITE_GOLDEN_DATA = 0; /* Overwrite golden data files? */
 size_t MINIMUM_MEMORY_SIZE_FAILURE = (size_t)-1; /* Minimum library memory allocation window that will fail. */
 size_t MAXIMUM_MEMORY_SIZE_FAILURE = (size_t)-1; /* Maximum library memory allocation window that will fail. */
-int32_t ALLOCATION_COUNT = 0;
 static const char *ARGV_0 = "[ALL]";
 static const char *XML_FILE_NAME=NULL;
 static char XML_PREFIX[256];
-
+static const char *SUMMARY_FILE = NULL;
 FILE *XML_FILE = NULL;
 /*-------------------------------------------*/
 
@@ -358,7 +365,7 @@ static void iterateTestsWithLevel ( const TestNode* root,
     } else {
     	log_testinfo_i("(%s) ", ARGV_0);
     }
-    ON_LINE = TRUE;  /* we are still on the line with the test name */
+    ON_LINE = true;  /* we are still on the line with the test name */
 
 
     if ( (mode == RUNTESTS) &&
@@ -377,17 +384,18 @@ static void iterateTestsWithLevel ( const TestNode* root,
         currentTest = root;
         INDENT_LEVEL = depth;  /* depth of subitems */
         ONE_ERROR=0;
-        HANGING_OUTPUT=FALSE;
+        HANGING_OUTPUT=false;
 #if SHOW_TIMES
         startTime = uprv_getRawUTCtime();
 #endif
+        strcpy(gTestName, pathToFunction);
         root->test();   /* PERFORM THE TEST ************************/
 #if SHOW_TIMES
         stopTime = uprv_getRawUTCtime();
 #endif
         if(HANGING_OUTPUT) {
           log_testinfo("\n");
-          HANGING_OUTPUT=FALSE;
+          HANGING_OUTPUT=false;
         }
         INDENT_LEVEL = depth-1;  /* depth of root */
         currentTest = NULL;
@@ -414,7 +422,7 @@ static void iterateTestsWithLevel ( const TestNode* root,
             }
           } else {
             /* put -- out at 30 sp. */
-            int spaces = FLAG_INDENT-(strlen(root->name)+depth);
+            int spaces = FLAG_INDENT - ((int)strlen(root->name) + depth);
             if(spaces<0) spaces=0;
             log_testinfo(" %*s[OK] ", spaces,"---");
           }
@@ -424,12 +432,12 @@ static void iterateTestsWithLevel ( const TestNode* root,
         if(timeDelta[0]) printf("%s", timeDelta);
 #endif
          
-        ON_LINE = TRUE; /* we are back on-line */
+        ON_LINE = true; /* we are back on-line */
     }
 
     INDENT_LEVEL = depth-1; /* root */
 
-    /* we want these messages to be at 0 indent. so just push the indent level breifly. */
+    /* we want these messages to be at 0 indent. so just push the indent level briefly. */
     if(mode==SHOWTESTS) {
     	log_testinfo("---%s%c\n",pathToFunction, nodeList[i]->test?' ':TEST_SEPARATOR );
     }
@@ -460,7 +468,7 @@ static void iterateTestsWithLevel ( const TestNode* root,
                   }
                 }
 
-    		ON_LINE=TRUE;
+    		ON_LINE=true;
     	}
 	}
     depth--;
@@ -512,7 +520,15 @@ runTests ( const TestNode *root )
 
     /*print out result summary*/
 
-    ON_LINE=FALSE; /* just in case */
+    ON_LINE=false; /* just in case */
+
+    if(knownList != NULL) {
+      if( udbg_knownIssue_print(knownList) ) {
+        fprintf(stdout, "(To run suppressed tests, use the -K option.) \n\n");
+      }
+      udbg_knownIssue_close(knownList);
+      knownList = NULL;
+    }
 
     if (ERROR_COUNT)
     {
@@ -523,6 +539,14 @@ runTests ( const TestNode *root )
         fprintf(stdout, " Errors in\n");
         for (i=0;i < ERRONEOUS_FUNCTION_COUNT; i++)
             fprintf(stdout, "[%s]\n",ERROR_LOG[i]);
+	if(SUMMARY_FILE != NULL) {
+	  FILE *summf = fopen(SUMMARY_FILE, "w");
+	  if(summf!=NULL) {
+	    for (i=0;i < ERRONEOUS_FUNCTION_COUNT; i++)
+	      fprintf(summf, "%s\n",ERROR_LOG[i]);
+	    fclose(summf);
+	  }
+	}
     }
     else
     {
@@ -617,7 +641,7 @@ static void go_offline_with_marker(const char *mrk) {
   
   if(ON_LINE) {
     log_testinfo(" {\n");
-    ON_LINE=FALSE;
+    ON_LINE=false;
   }
   
   if(!HANGING_OUTPUT || wasON_LINE) {
@@ -654,7 +678,7 @@ static void first_line_test() {
 
 static void vlog_err(const char *prefix, const char *pattern, va_list ap)
 {
-    if( ERR_MSG == FALSE){
+    if( ERR_MSG == false){
         return;
     }
     fputs("!", stdout); /* col 1 - bang */
@@ -673,6 +697,29 @@ static void vlog_err(const char *prefix, const char *pattern, va_list ap)
     GLOBAL_PRINT_COUNT++;
 }
 
+static UBool vlog_knownIssue(const char *ticket, const char *pattern, va_list ap)
+{
+    char buf[2048];
+    UBool firstForTicket;
+    UBool firstForWhere;
+
+    if(NO_KNOWN) return false;
+    if(pattern==NULL) pattern="";
+
+    vsprintf(buf, pattern, ap);
+    knownList = udbg_knownIssue_open(knownList, ticket, gTestName, buf,
+                                     &firstForTicket, &firstForWhere);
+
+    if(firstForTicket || firstForWhere) {
+      log_info("(Known issue %s) %s\n", ticket, buf);
+    } else {
+      log_verbose("(Known issue %s) %s\n", ticket, buf);
+    }
+
+    return true;
+}
+
+
 void T_CTEST_EXPORT2
 vlog_info(const char *prefix, const char *pattern, va_list ap)
 {
@@ -722,7 +769,7 @@ static void log_testinfo(const char *pattern, ...)
 
 static void vlog_verbose(const char *prefix, const char *pattern, va_list ap)
 {
-    if ( VERBOSITY == FALSE )
+    if ( VERBOSITY == false )
         return;
 
     first_line_verbose();
@@ -760,6 +807,13 @@ log_err(const char* pattern, ...)
     vlog_err(NULL, pattern, ap);
 }
 
+UBool T_CTEST_EXPORT2
+log_knownIssue(const char *ticket, const char *pattern, ...) {
+  va_list ap;
+  va_start(ap, pattern);
+  return vlog_knownIssue(ticket, pattern, ap);
+}
+
 void T_CTEST_EXPORT2
 log_err_status(UErrorCode status, const char* pattern, ...)
 {
@@ -839,18 +893,22 @@ log_data_err(const char* pattern, ...)
 static int traceFnNestingDepth = 0;
 U_CDECL_BEGIN
 static void U_CALLCONV TraceEntry(const void *context, int32_t fnNumber) {
+    (void)context; // suppress compiler warnings about unused variable
     char buf[500];
-    utrace_format(buf, sizeof(buf), traceFnNestingDepth*3, "%s() enter.\n", utrace_functionName(fnNumber));    buf[sizeof(buf)-1]=0;  
+    utrace_format(buf, sizeof(buf), traceFnNestingDepth*3, "%s() enter.\n", utrace_functionName(fnNumber));
+    buf[sizeof(buf)-1]=0;  
     fputs(buf, stdout);
     traceFnNestingDepth++;
 }   
  
-static void U_CALLCONV TraceExit(const void *context, int32_t fnNumber, const char *fmt, va_list args) {    char buf[500];
-    
+static void U_CALLCONV TraceExit(const void *context, int32_t fnNumber, const char *fmt, va_list args) {
+    (void)context; // suppress compiler warnings about unused variable
+    char buf[500];
     if (traceFnNestingDepth>0) {
         traceFnNestingDepth--; 
     }
-    utrace_format(buf, sizeof(buf), traceFnNestingDepth*3, "%s() ", utrace_functionName(fnNumber));    buf[sizeof(buf)-1]=0;
+    utrace_format(buf, sizeof(buf), traceFnNestingDepth*3, "%s() ", utrace_functionName(fnNumber));
+    buf[sizeof(buf)-1]=0;
     fputs(buf, stdout);
     utrace_vformat(buf, sizeof(buf), traceFnNestingDepth*3, fmt, args);
     buf[sizeof(buf)-1]=0;
@@ -860,6 +918,10 @@ static void U_CALLCONV TraceExit(const void *context, int32_t fnNumber, const ch
 
 static void U_CALLCONV TraceData(const void *context, int32_t fnNumber,
                           int32_t level, const char *fmt, va_list args) {
+    // suppress compiler warnings about unused variables
+    (void)context;  
+    (void)fnNumber;
+    (void)level;
     char buf[500];
     utrace_vformat(buf, sizeof(buf), traceFnNestingDepth*3, fmt, args);
     buf[sizeof(buf)-1]=0;
@@ -868,16 +930,17 @@ static void U_CALLCONV TraceData(const void *context, int32_t fnNumber,
 }
 
 static void *U_CALLCONV ctest_libMalloc(const void *context, size_t size) {
+    (void)context; // suppress compiler warnings about unused variable
     /*if (VERBOSITY) {
         printf("Allocated %ld\n", (long)size);
     }*/
     if (MINIMUM_MEMORY_SIZE_FAILURE <= size && size <= MAXIMUM_MEMORY_SIZE_FAILURE) {
         return NULL;
     }
-    umtx_atomic_inc(&ALLOCATION_COUNT);
     return malloc(size);
 }
 static void *U_CALLCONV ctest_libRealloc(const void *context, void *mem, size_t size) {
+    (void)context; // suppress compiler warnings about unused variable
     /*if (VERBOSITY) {
         printf("Reallocated %ld\n", (long)size);
     }*/
@@ -885,16 +948,10 @@ static void *U_CALLCONV ctest_libRealloc(const void *context, void *mem, size_t
         /*free(mem);*/ /* Realloc doesn't free on failure. */
         return NULL;
     }
-    if (mem == NULL) {
-        /* New allocation. */
-        umtx_atomic_inc(&ALLOCATION_COUNT);
-    }
     return realloc(mem, size);
 }
 static void U_CALLCONV ctest_libFree(const void *context, void *mem) {
-    if (mem != NULL) {
-        umtx_atomic_dec(&ALLOCATION_COUNT);
-    }
+    (void)context; // suppress compiler warnings about unused variable
     free(mem);
 }
 
@@ -902,11 +959,10 @@ int T_CTEST_EXPORT2
 initArgs( int argc, const char* const argv[], ArgHandlerPtr argHandler, void *context)
 {
     int                i;
-    int                doList = FALSE;
-	int                argSkip = 0;
+    int                argSkip = 0;
 
-    VERBOSITY = FALSE;
-    ERR_MSG = TRUE;
+    VERBOSITY = false;
+    ERR_MSG = true;
 
     ARGV_0=argv[0];
 
@@ -924,11 +980,11 @@ initArgs( int argc, const char* const argv[], ArgHandlerPtr argHandler, void *co
         }
         else if (strcmp( argv[i], "-v" )==0 || strcmp( argv[i], "-verbose")==0)
         {
-            VERBOSITY = TRUE;
+            VERBOSITY = true;
         }
         else if (strcmp( argv[i], "-l" )==0 )
         {
-            doList = TRUE;
+            /* doList = true; */
         }
         else if (strcmp( argv[i], "-e1") == 0)
         {
@@ -938,9 +994,17 @@ initArgs( int argc, const char* const argv[], ArgHandlerPtr argHandler, void *co
         {
             QUICK = 0;
         }
+        else if (strcmp( argv[i], "-K") ==0)
+        {
+            NO_KNOWN = 1;
+        }
+        else if (strncmp( argv[i], "-E",2) ==0)
+        {
+	    SUMMARY_FILE=argv[i]+2;
+        }
         else if (strcmp( argv[i], "-w") ==0)
         {
-            WARN_ON_MISSING_DATA = TRUE;
+            WARN_ON_MISSING_DATA = true;
         }
         else if (strcmp( argv[i], "-m") ==0)
         {
@@ -974,7 +1038,7 @@ initArgs( int argc, const char* const argv[], ArgHandlerPtr argHandler, void *co
         }
         else if(strcmp( argv[i], "-n") == 0 || strcmp( argv[i], "-no_err_msg") == 0)
         {
-            ERR_MSG = FALSE;
+            ERR_MSG = false;
         }
         else if (strcmp( argv[i], "-r") == 0)
         {
@@ -1007,6 +1071,9 @@ initArgs( int argc, const char* const argv[], ArgHandlerPtr argHandler, void *co
         else if (strcmp( argv[i], "-t_oc") == 0) {
             ICU_TRACE = UTRACE_OPEN_CLOSE;
         }
+        else if (strcmp( argv[i], "-G") == 0) {
+            WRITE_GOLDEN_DATA = 1;
+        }
         else if (strcmp( argv[i], "-h" )==0 || strcmp( argv[i], "--help" )==0)
         {
             help( argv[0] );
@@ -1041,8 +1108,8 @@ runTestRequest(const TestNode* root,
      */
     const TestNode*    toRun;
     int                i;
-    int                doList = FALSE;
-    int                subtreeOptionSeen = FALSE;
+    int                doList = false;
+    int                subtreeOptionSeen = false;
 
     int                errorCount = 0;
 
@@ -1069,40 +1136,40 @@ runTestRequest(const TestNode* root,
                 return -1;
             }
 
-            ON_LINE=FALSE; /* just in case */
+            ON_LINE=false; /* just in case */
 
-            if( doList == TRUE)
+            if( doList == true)
                 showTests(toRun);
             else
                 runTests(toRun);
 
-            ON_LINE=FALSE; /* just in case */
+            ON_LINE=false; /* just in case */
 
             errorCount += ERROR_COUNT;
 
-            subtreeOptionSeen = TRUE;
+            subtreeOptionSeen = true;
         } else if ((strcmp( argv[i], "-a") == 0) || (strcmp(argv[i],"-all") == 0)) {
-            subtreeOptionSeen=FALSE;
+            subtreeOptionSeen=false;
         } else if (strcmp( argv[i], "-l") == 0) {
-            doList = TRUE;
+            doList = true;
         }
         /* else option already handled by initArgs */
     }
 
-    if( subtreeOptionSeen == FALSE) /* no other subtree given, run the default */
+    if( subtreeOptionSeen == false) /* no other subtree given, run the default */
     {
-        ON_LINE=FALSE; /* just in case */
-        if( doList == TRUE)
+        ON_LINE=false; /* just in case */
+        if( doList == true)
             showTests(toRun);
         else
             runTests(toRun);
-        ON_LINE=FALSE; /* just in case */
+        ON_LINE=false; /* just in case */
 
         errorCount += ERROR_COUNT;
     }
     else
     {
-        if( ( doList == FALSE ) && ( errorCount > 0 ) )
+        if( ( doList == false ) && ( errorCount > 0 ) )
             printf(" Total errors: %d\n", errorCount );
     }
 
@@ -1131,6 +1198,7 @@ static void help ( const char *argv0 )
     printf("    -v  To turn ON verbosity(same as -verbose)\n");
     printf("    -x file.xml   Write junit format output to file.xml\n");
     printf("    -h  To print this message\n");
+    printf("    -K  to turn OFF suppressing known issues\n");
     printf("    -n  To turn OFF printing error messages\n");
     printf("    -w  Don't fail on data-loading errs, just warn. Useful if\n"
            "        user has reduced/changed the common set of ICU data \n");
@@ -1139,6 +1207,7 @@ static void help ( const char *argv0 )
     printf("    -m n[-q] Min-Max memory size that will cause an allocation failure.\n");
     printf("        The default is the maximum value of size_t. Max is optional.\n");
     printf("    -r  Repeat tests after calling u_cleanup \n");
+    printf("    -G  Write golden data files \n");
     printf("    [/subtest]  To run a subtest \n");
     printf("    eg: to run just the utility tests type: cintltest /tsutil) \n");
 }
@@ -1158,6 +1227,8 @@ getTestOption ( int32_t testOption ) {
             return ERR_MSG;
         case ICU_TRACE_OPTION:
             return ICU_TRACE;
+        case WRITE_GOLDEN_DATA_OPTION:
+            return WRITE_GOLDEN_DATA;
         default :
             return 0;
     }
@@ -1183,8 +1254,10 @@ setTestOption ( int32_t testOption, int32_t value) {
             REPEAT_TESTS = value;
             break;
         case ICU_TRACE_OPTION:
-            ICU_TRACE = value;
+            ICU_TRACE = (UTraceLevel)value;
             break;
+        case WRITE_GOLDEN_DATA_OPTION:
+            WRITE_GOLDEN_DATA = value;
         default :
             break;
     }
@@ -1213,13 +1286,13 @@ ctest_xml_init(const char *rootName) {
     fprintf(stderr," Error: couldn't open XML output file %s\n", XML_FILE_NAME);
     return 1;
   }
-  while(*rootName&&!isalnum(*rootName)) {
+  while(*rootName&&!isalnum((int)*rootName)) {
     rootName++;
   }
   strcpy(XML_PREFIX,rootName);
   {
     char *p = XML_PREFIX+strlen(XML_PREFIX);
-    for(p--;*p&&p>XML_PREFIX&&!isalnum(*p);p--) {
+    for(p--;*p&&p>XML_PREFIX&&!isalnum((int)*p);p--) {
       *p=0;
     }
   }
@@ -1244,10 +1317,10 @@ ctest_xml_fini(void) {
 
 int32_t
 T_CTEST_EXPORT2
-ctest_xml_testcase(const char *classname, const char *name, const char *time, const char *failMsg) {
+ctest_xml_testcase(const char *classname, const char *name, const char *timeSeconds, const char *failMsg) {
   if(!XML_FILE) return 0;
 
-  fprintf(XML_FILE, "\t<testcase classname=\"%s:%s\" name=\"%s:%s\" time=\"%s\"", XML_PREFIX, classname, XML_PREFIX, name, time);
+  fprintf(XML_FILE, "\t<testcase classname=\"%s:%s\" name=\"%s:%s\" time=\"%s\"", XML_PREFIX, classname, XML_PREFIX, name, timeSeconds);
   if(failMsg) {
     fprintf(XML_FILE, ">\n\t\t<failure type=\"err\" message=\"%s\"/>\n\t</testcase>\n", failMsg);
   } else {
