@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 **********************************************************************
 * Copyright (c) 2002-2014, International Business Machines
@@ -131,7 +133,7 @@ public:
 class T_CTEST_EXPORT_API UPerfTest {
 public:
     UBool run();
-    UBool runTest( char* name = NULL, char* par = NULL ); // not to be overidden
+    UBool runTest( char* name = NULL, char* par = NULL ); // not to be overridden
         
     virtual void usage( void ) ;
     
@@ -156,7 +158,7 @@ protected:
     void init(UOption addOptions[], int32_t addOptionsCount,
               UErrorCode& status);
 
-    virtual UPerfFunction* runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ); // overide !
+    virtual UPerfFunction* runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ); // override !
 
     virtual UBool runTestLoop( char* testname, char* par );
 
