@@ -48,6 +48,7 @@
 #   define ICU_ENTRY_OFFSET 0
 #endif
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "unicode/putil.h"
@@ -96,7 +97,7 @@ static UOption options[]={
 #define CALL_WRITEOBJECT    'o'
 extern int
 main(int argc, char* argv[]) {
-    UBool verbose = TRUE;
+    UBool verbose = true;
     char writeCode;
 
     U_MAIN_INIT_ARGS(argc, argv);
@@ -166,7 +167,7 @@ main(int argc, char* argv[]) {
             /* TODO: remove writeCode=&writeCCode; */
         }
         if (options[kOptQuiet].doesOccur) {
-            verbose = FALSE;
+            verbose = false;
         }
         while(--argc) {
             filename=getLongPathname(argv[argc]);
